<?php
class Post_Labels {
	private $post_api;
	private $post_emails;
	private $post_settings;
	private $post_configs;
	private $send_methods = array();

	public function __construct() {
		$this->post_api = new Post_Api();
		$this->post_emails = new Post_Emails();
		$this->post_configs = Post_Core::get_configs();
		$this->post_settings = Post_Core::get_settings();

		foreach ($this->post_configs['method_params'] as $method_name => $method_values) {
			if (!$method_values['is_shipping_method']) continue;
			$this->send_methods[$method_values['key']] = 'post_' . $method_values['key'];
		}
	}

	public function create_parcel($orderIds = array(), $regenerate = false) {
		$url = 'https://express.pasts.lv/parcelsApi/create';
		$secret = $this->post_settings['api_user'];
		$hs_code = $this->post_settings['hs_code'];
		$origin_country_id = $this->post_settings['origin_country_id'];
		$error = array();
		$countries = $this->getCountries();

		foreach ($orderIds as $orderId) {
			$barcode = get_post_meta($orderId, Post_Core::get_configs()['meta_keys']['barcode'], true);
			if (!$barcode  || $regenerate) {				
				$wc_order = wc_get_order((int)$orderId);
				$send_method = $this->post_api->get_send_method($wc_order);
				$terminal_id = get_post_meta($orderId, Post_Core::get_configs()['meta_keys']['terminal_id'], true);
				$phone = $wc_order->get_shipping_phone();
				if (!$phone) {
					$phone = $wc_order->get_billing_phone();
				}

				// Cash on delivery data
				if ($wc_order->get_payment_method() == 'cod') {
					$cod = array(
						"insured" => 1,
						"insurance_sum" => $wc_order->get_total(),            
						"post_payment" => 1,
						"post_payment_sum" => $wc_order->get_total(),
						"post_payment_aim" => "Order " . $orderId,
					);
				} else {
					$cod = array();
				}
			
				$country = mb_strtoupper($wc_order->get_shipping_country());
				// Make 1 variable that stores country ID for LV, LT or EE
				switch ($country) {
					case "LV":
						$baltic_country_id = $countries['LV']['id'] ?? '7';
						break;
					case "LT":
						$baltic_country_id = $countries['LT']['id'] ?? '2';
						break;
					case "EE":
						$baltic_country_id = $countries['EE']['id'] ?? '1';
						break;
					default:
						$baltic_country_id = '';
				}

				if ($country == 'LV') {
					$zipcode = $wc_order->get_shipping_postcode();
				}else{
					$zipcode = preg_replace('/[^0-9]/', '', $wc_order->get_shipping_postcode());
				}

				if ($send_method == 'pt') {
					$shipment_data = array(	
						$orderId =>	array(		
							"name_surname" => $wc_order->get_shipping_first_name() . ' ' . $wc_order->get_shipping_last_name(),
							"phone" => $phone,
							"email" => $wc_order->get_billing_email(),
							"country_id" => $baltic_country_id,
							"comments" => $wc_order->get_customer_note(),
						)
					);
					$shipment_data[$orderId] = array_merge($shipment_data[$orderId], $cod);

					if ($country == 'LV') {
						$shipment_data[$orderId]['type'] = "Ie";						
						$shipment_data[$orderId]['station_id'] = $terminal_id;
					} elseif ($country == 'LT') {
						$terminals_json_file_dir = POST_DIR . '/' . "locations_lt.json";
						$terminals_file = fopen($terminals_json_file_dir, "r");
						$terminals = fread($terminals_file, filesize($terminals_json_file_dir) + 10);
						fclose($terminals_file);
						$terminals = json_decode($terminals, true);
						if (is_array($terminals) && $terminal_id) {
							foreach ($terminals as $terminal) {
								if ($terminal['id'] == $terminal_id) {
									$postcode = str_replace("LT-", "", $terminal['postal_code']);
									$city = $terminal['city'];
									break;
								}
							}
						}

						$shipment_data[$orderId]['type'] = "Be";
						$shipment_data[$orderId]['terminal_id'] = $terminal_id;
						$shipment_data[$orderId]['size'] = "Medium";
						$shipment_data[$orderId]['zipcode'] = $postcode ?: $wc_order->get_shipping_postcode();
						$shipment_data[$orderId]['city'] = $city ?: $wc_order->get_shipping_city();

						// Multiparcel
						foreach ($wc_order->get_items() as $multi) {

						}
					}else{
						Post_Helper::add_msg($orderId . ' - ' . __('Can\'t use this shipping method with selected country', 'post'), 'error');
						wp_safe_redirect(wp_get_referer());
					}
				} elseif ($send_method == 'c') {
					if ($country == 'LT' || $country == 'EE') {
						$country_type = 'Be';
					}elseif($country == 'LV' ) {
						$country_type = 'Ie';
					}else{
						$country_type = 'Ems';
						$country_number = $countries[$country]['id'] ?? '';						
					}

					$product_data = array();
					foreach ($wc_order->get_items() as $item_id => $item) {
						if ($item->get_product()->get_weight() !== '') {
							$weight = $item->get_product()->get_weight();
						} else {
							Post_Helper::add_msg($orderId . ' - ' . __('Pakas satura svars nevar būt tukšs', 'post'), 'error');
							wp_safe_redirect(wp_get_referer());
						};

						$product_data[] = array(
							"hs_code" => $hs_code,
							"origin_country_id" => $origin_country_id,
							"title" => $item->get_name(),
							"amount" => $item->get_quantity(),
							"value" => $item->get_total(),
							"weight" => $weight,
						);
					}

					$shipment_data = array(
						$orderId =>	array(
							"type" => $country_type,
							"name_surname" => $wc_order->get_shipping_first_name() . ' ' . $wc_order->get_shipping_last_name(),
							"phone" => $phone,
							"email" => $wc_order->get_billing_email(),
							"country_id" => $baltic_country_id ?: $country_number,
							"city"=> $wc_order->get_shipping_city(),
							"street" => $wc_order->get_shipping_address_1(),	
							"zipcode" => $zipcode,
							"comments" => $wc_order->get_customer_note(),
							"package_contents" => "Prece",
							"content_currency" => "EUR",
							"ParcelContent" => $product_data
						)
					);
					$shipment_data[$orderId] = array_merge($shipment_data[$orderId], $cod);

					if($country == 'US') {
						$shipment_data[$orderId]['zip_state'] = $wc_order->get_shipping_state();
						$shipment_data[$orderId]['house'] = '.';
					}
					if($country == 'RU') {
						$shipment_data[$orderId]['house'] = '.';
					}

				} elseif ($send_method == 'po') {
					if($country == 'LV' || $country == 'LT') {
						$shipment_data = array(
							$orderId =>	array(
								"name_surname" => $wc_order->get_shipping_first_name() . ' ' . $wc_order->get_shipping_last_name(),
								"phone" => $phone,
								"email" => $wc_order->get_billing_email(),
								"country_id" => $baltic_country_id,
								"city"=> $wc_order->get_shipping_city(),
								"street" => $wc_order->get_shipping_address_1(),	
								"zipcode" => $zipcode,
								"comments" => $wc_order->get_customer_note(),
								"pickup" => 1,
							)
						);
						$shipment_data[$orderId] = array_merge($shipment_data[$orderId], $cod);

						// Different data for LV and LT
						switch ($country) {
							case "LV":
								$terminals_json_file_dir = POST_DIR . '/' . "locations_office.json";
								$terminals_file = fopen($terminals_json_file_dir, "r");
								$terminals = fread($terminals_file, filesize($terminals_json_file_dir) + 10);
								fclose($terminals_file);
								$terminals = json_decode($terminals, true);
								$parcel_terminals = '';
								if (is_array($terminals) && $terminal_id) {
									foreach ($terminals as $terminal) {
										if ($terminal['id'] == $terminal_id) {
											$terminal_postcode = 'LV-' . $terminal['postcode'];
										}
									}
								}
								$shipment_data[$orderId]['type'] = "Ie";
								$shipment_data[$orderId]['pickup_zipcode'] = $terminal_postcode;
								$shipment_data[$orderId]['sms_invite'] = 1;
								break;
							case "LT":
								$shipment_data[$orderId]['type'] = "Be";
								$shipment_data[$orderId]['pickup_zipcode'] = $terminal_id;
								break;
							default:
						}
					}else{
						Post_Helper::add_msg($orderId . ' - ' . __('Can\'t use this shipping method with selected country', 'post'), 'error');
						wp_safe_redirect(wp_get_referer());
					}
				} elseif ($send_method == 'ck') {
					if($country == 'LV') {
						$shipment_data = array(
							$orderId =>	array(
								"type" => "Ie",
								 "name_surname" => $wc_order->get_shipping_first_name() . ' ' . $wc_order->get_shipping_last_name(),
								 "phone" => $phone,
								 "email" => $wc_order->get_billing_email(),
								 "country_id" => $baltic_country_id,
								 "comments" => $wc_order->get_customer_note() ,
								 "statoil_id" => $terminal_id,
							)
						);
						$shipment_data[$orderId] = array_merge($shipment_data[$orderId], $cod);
					}else{
						Post_Helper::add_msg($orderId . ' - ' . __('Can\'t use this shipping method with selected country', 'post'), 'error');
						wp_safe_redirect(wp_get_referer());
					}
				}

				$data = array(
					"secret" => $secret,
					"parcels" => $shipment_data,
				);

				$response = $this->send($url, $data);

				//Debug
				$data['url'] = $url;
				Post_Debug::debug_write($data, "CREATE PARCEL REQUEST");
				Post_Debug::debug_write($response, "CREATE PARCEL RESPONSE");

				// Checks response for errors. They can be in multiple levels
				if (!is_array($response)) {
					$response = json_decode($response, true);
					$err_keys = array('phone', 'zipcode', 'pickup_zipcode', 'zip_state', 'country_id');
					if(isset($response['error'])) {
						$error[$orderId] = $response['error'];
						update_post_meta($orderId, Post_Core::get_configs()['meta_keys']['error'], $response['error']);
						update_post_meta($orderId, '_post_barcode', '');
					}elseif(!isset($response[$orderId])) {
						$error[$orderId] = $response['error'];
						update_post_meta($orderId, Post_Core::get_configs()['meta_keys']['error'], $response['error']);
						update_post_meta($orderId, '_post_barcode', '');
					}else{
						if(is_array($response[$orderId])) {
							$err_msg = '';
							foreach($err_keys as $key) {
								if(array_key_exists($key, $response[$orderId])) {
									$err_msg .= $response[$orderId][$key] . '; ';
								}
							}

							if (isset($response[$orderId]['ParcelContent'])) {
								$err_msg .= json_encode($response[$orderId]['ParcelContent'], JSON_UNESCAPED_UNICODE) . '; ';
							}elseif(empty($err_msg)){
								$err_msg = json_encode($response[$orderId], JSON_UNESCAPED_UNICODE);
							}

							$error[$orderId] = $err_msg;
							update_post_meta($orderId, Post_Core::get_configs()['meta_keys']['error'], $err_msg);
							update_post_meta($orderId, '_post_barcode', '');
						} else {
							update_post_meta($orderId, Post_Core::get_configs()['meta_keys']['error'], '');

							if (count($response) > 1) {
								update_post_meta($orderId, '_post_barcode', implode(',', $response));
							} else {
								update_post_meta($orderId, '_post_barcode', $response[$orderId]);
							}
						}					
					}
				}
			}
		}

		return $error;
	}

	public function print_labels($orderIds = array(), $download = true, $regenerate = false) {
		$error = array();
		$error = $this->create_parcel($orderIds, $regenerate);

		if (empty($error)) {
			$label_barcodes = $sticker_barcodes = [];
			$time = time();
			foreach ($orderIds as $orderId) {
				$wc_order = wc_get_order((int)$orderId);
				$send_method = $this->post_api->get_send_method($wc_order);
				$country = mb_strtoupper($wc_order->get_shipping_country());
				$barcode = get_post_meta($orderId, Post_Core::get_configs()['meta_keys']['barcode'], true);

				if (($send_method == 'c') && ($country != 'LT' && $country != 'LV')) {
					$label_barcodes = array_merge($label_barcodes, explode(',', $barcode));
				}else{
					$sticker_barcodes = array_merge($sticker_barcodes, explode(',', $barcode));
				}
			}

			if (!empty($label_barcodes)) {
				$url = 'https://express.pasts.lv/parcelsDocumentsApi/labels';
				$data = array(
					"secret" => $this->post_settings['api_user'],
					"parcels" => $label_barcodes,
				);

				//Debug
				$data['url'] = $url;
				Post_Debug::debug_write($data, "LABEL REQUEST");
				
				$label_response = $this->send($url, $data);
				//Post_Debug::debug_write($label_response, "LABEL RESPONSE");
			}
			if (!empty($sticker_barcodes)) {
				$url = 'https://express.pasts.lv/parcelsDocumentsApi/stickers';
				$data = array(
					"secret" => $this->post_settings['api_user'],
					"size" => "150x100",
					"parcels" => $sticker_barcodes,
				);

				//Debug
				$data['url'] = $url;
				Post_Debug::debug_write($data, "STICKER REQUEST");

				$sticker_response = $this->send($url, $data);
				//Post_Debug::debug_write($sticker_response, "STICKER RESPONSE");
			}

			if(!empty($label_response) XOR !empty($sticker_response)) {
				$response = !empty($label_response) ? $label_response : $sticker_response;
				$resp = json_decode($response, true);
				$name = !empty($label_response) ? 'labels_'.$time.'.pdf' : 'stickers_'.$time.'.pdf';

				if (!is_array($response) && json_last_error() != 0) {
					ob_start();
					header('Content-Description: File Transfer');
					header('Content-type: application/pdf');
					header('Content-Disposition: attachment; filename='.$name);
					header("Expires: 0");
					header( "refresh:1;url=".$_SERVER['HTTP_REFERER'] ?? '' ."" );
					header("Cache-Control: must-revalidate");
					header("Pragma: public");
					header('Content-Transfer-Encoding: binary');
					header('Content-Length: ' . strlen($response));
					ob_end_flush();

					echo $response;
				} else {
					$err_msg = sprintf(__('Can\'t generate %s. API response: ', 'post'), 'labels');
					Post_Helper::add_msg($err_msg . json_encode($response, JSON_UNESCAPED_UNICODE), 'error');
					wp_safe_redirect(wp_get_referer());
				}
			}elseif(!empty($label_response) && !empty($sticker_response)){
				$label_resp = json_decode($label_response, true);
				$sticker_resp = json_decode($sticker_response, true);
				if (!is_array($label_response) && !is_array($sticker_response) && json_last_error() != 0) {
					//Output files as ZIP
					if(class_exists('ZipArchive')) {						
						$zip = new ZipArchive();
						$zipname = 'labels_'.$time.".zip"; // Zip name
						$zip->open($zipname,  ZipArchive::CREATE);
						$zip->addFromString('labels_'.$time.'.pdf', $label_response);
						$zip->addFromString('stickers_'.$time.'.pdf', $sticker_response);
						$zip->close();

						ob_start();
						header('Content-Description: File Transfer');
						header('Cache-Control: public, must-revalidate, max-age=0'); // HTTP/1.1
						header('Pragma: public');
						header('Expires: 0');
						header("refresh:1;url=".$_SERVER['HTTP_REFERER'] ?? '' ."" );
						header('Content-Type: application/zip');
						header('Content-disposition: attachment; filename='.$zipname);
						header('Content-Length: ' . filesize($zipname));
						ob_end_flush();
						readfile($zipname);
					}else{
						Post_Helper::add_msg(__('PHP ZipArchive library not installed https://www.php.net/manual/en/class.ziparchive.php', 'post'), 'error');
						wp_safe_redirect(wp_get_referer());
					}
				} else {
					$resp = is_array($label_response) ? json_encode($label_response, JSON_UNESCAPED_UNICODE) : json_encode($sticker_response, JSON_UNESCAPED_UNICODE);
					$err_msg = sprintf(__('Can\'t generate %s. API response: ', 'post'), 'labels');
					Post_Helper::add_msg($err_msg . $resp, 'error');
					wp_safe_redirect(wp_get_referer());
				}
			}else{
				Post_Helper::add_msg(__('Can\'t generate labels. Empty API response', 'post'), 'error');
				wp_safe_redirect(wp_get_referer());		
			}
		}else{
			foreach($error as $id=>$err) {
				Post_Helper::add_msg('Order #' . $id . ' - ' . $err, 'error');
			}
			wp_safe_redirect(wp_get_referer());
		}
	}

	public function clean($string) {
		return str_replace('"', "'", $string);
	}

	public static function post_call_courier_actions() {
		$post_api = new Post_Api();
		$callCarrierReturn = $post_api->call_courier(intval($_GET['call_quantity']));

		if (!empty($callCarrierReturn['msg'])) {
			Post_Helper::add_msg(__("Post courier called", 'post'), 'post-notice');
		} else {		
			Post_Helper::add_msg(__("There was an error calling Post courier. Error: " . $callCarrierReturn['err'], 'post'), 'error');
		}
		wp_safe_redirect(wp_get_referer());
	}

	public function send($url, $data = null, $checkCert = true)	{
		$curl = curl_init();
		curl_setopt($curl, CURLINFO_HEADER_OUT, true);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
		//curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, $checkCert);
		//curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, $checkCert);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($curl, CURLOPT_POSTREDIR, true);
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_POST, true);
		if (is_string($data)) {
			$data = json_decode($data, true);
		}
		if (empty($data) || !is_array($data)) {
			return false;
		}
		// Content-Type: application/x-www-form-urlencoded
		$data = http_build_query($data);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
		$response = curl_exec($curl);
		$errCode = curl_errno($curl);
		$errText = curl_error($curl);
		curl_close($curl);
		if (!empty($errCode)) {
			$response = array(
				'code' => $errCode,
				'text' => $errText
			);
		}
		return $response;
	}

	public function getCountries(){						
		$json = @file_get_contents('https://ftp.webdev.lv/pasts/countries.json');
		$results = json_decode($json, true);
		$countries = array();
	
		foreach($results as $result){
			if(isset($result['Country'])){
				$countries[$result['Country']['iso2']] = $result['Country'];
			} 			
		}
		
		return $countries;
	}
}
