<?php
class Post_Manifest {
	public static function load_admin_scripts() {
		$folder_css = '/assets/css/';
		$folder_js = '/assets/js/';

		wp_enqueue_style('post_admin_manifest', plugins_url($folder_css . 'post_admin_manifest.css', Post_Core::$main_file_path, array(), POST_VERSION));
		wp_enqueue_style('bootstrap-datetimepicker', plugins_url($folder_js . 'datetimepicker/bootstrap-datetimepicker.min.css', Post_Core::$main_file_path));

		wp_enqueue_script('moment', plugins_url($folder_js . 'moment.min.js', Post_Core::$main_file_path), array(), null, true);
		wp_enqueue_script('bootstrap-datetimepicker', plugins_url($folder_js . 'datetimepicker/bootstrap-datetimepicker.min.js', Post_Core::$main_file_path), array('jquery', 'moment'), null, true);
		wp_enqueue_script('post_helper', plugins_url($folder_js . 'post_helper.js', Post_Core::$main_file_path), array(), null, true);
		wp_enqueue_script('post_manifest', plugins_url($folder_js . 'post_manifest.js', Post_Core::$main_file_path), array(), null, true);

		wp_localize_script('post_manifest', 'postglobals', array(
			'cookie_checked_list' => 'post_checked',
		));

		wp_localize_script('post_manifest', 'posttext', array(
			'alert_select_orders' => __('Please select orders', 'post'),
		));
	}

	public static function register_menu_pages() {
		add_submenu_page(
			'woocommerce',
			__('Expresspasts shipping', 'post'),
			__('Expresspasts shipping', 'post'),
			'manage_woocommerce',
			'post-manifest',
			'Post_Manifest::manifest_page',
			1
		);
	}

	public static function manifest_page() {
		include_once(POST_DIR . 'manifest_page.php');
	}

	/**
	 * Handle a custom query variable to get orders.
	 * @param array $query - Args for WP_Query.
	 * @param array $query_vars - Query vars from WC_Order_Query.
	 * @return array modified $query
	 */
	public static function handle_custom_query_var($query, $query_vars) {
		$configs_meta = Post_Core::get_configs('meta_keys');
		if (!empty($query_vars['post_method'])) {
			$query['meta_query'][] = array(
				'key' => $configs_meta['method'],
				'value' => $query_vars['post_method'] //esc_attr( $query_vars['post_method'] ),
			);
		}

		if (isset($query_vars['post_barcode'])) {
			$query['meta_query'][] = array(
				'key' => $configs_meta['barcode'],
				'value' => $query_vars['post_barcode'],
				'compare' => 'LIKE'
			);
		}

		if (isset($query_vars['post_customer'])) {
			$query['meta_query'][] = array(
				'relation' => 'OR',
				array(
					'key' => '_billing_first_name',
					'value' => $query_vars['post_customer'],
					'compare' => 'LIKE'
				),
				array(
					'key' => '_billing_last_name',
					'value' => $query_vars['post_customer'],
					'compare' => 'LIKE'
				)
			);
		}

		if (isset($query_vars['post_manifest'])) {
			$query['meta_query'][] = array(
				'key' => $configs_meta['manifest_date'],
				'compare' => ($query_vars['post_manifest'] ? 'EXISTS' : 'NOT EXISTS'),
			);
		}

		if (isset($query_vars['post_manifest_date'])) {
			$filter_by_date = false;
			if ($query_vars['post_manifest_date'][0] && $query_vars['post_manifest_date'][1]) {
				$filter_by_date = array(
					'key' => $configs_meta['manifest_date'],
					'value' => $query_vars['post_manifest_date'],
					'compare' => 'BETWEEN'
				);
			} elseif ($query_vars['post_manifest_date'][0] && !$query_vars['post_manifest_date'][1]) {
				$filter_by_date = array(
					'key' => $configs_meta['manifest_date'],
					'value' => $query_vars['post_manifest_date'][0],
					'compare' => '>='
				);
			} elseif (!$query_vars['post_manifest_date'][0] && $query_vars['post_manifest_date'][1]) {
				$filter_by_date = array(
					'key' => $configs_meta['manifest_date'],
					'value' => $query_vars['post_manifest_date'][1],
					'compare' => '<='
				);
			}

			if ($filter_by_date) {
				$query['meta_query'][] = $filter_by_date;
			}
		}

		return $query;
	}

	public static function page_params() {
		return array(
			'strings' => array(
				'all_orders' => __('All orders', 'post'),
				'new_orders' => __('New orders', 'post'),
				'completed_orders' => __('Completed orders', 'post'),
			),
			'filter_keys' => array('customer', 'status', 'barcode', 'id', 'start_date', 'end_date'),
			'per_page' => 25,
		);
	}

	public static function page_make_link($args) {
		$query_args = array('page' => 'post-manifest');
		$query_args = array_merge($query_args, $args);
		return add_query_arg($query_args, admin_url('/admin.php'));
	}

	public static function page_get_orders() {
		$configs = Post_Core::get_configs();
		$page_params = self::page_params();

		$paged = (isset($_GET['paged'])) ? filter_input(INPUT_GET, 'paged') : 1;
		$action = (isset($_GET['action'])) ? filter_input(INPUT_GET, 'action') : 'all_orders';

		$filters = array();
		foreach ($page_params['filter_keys'] as $filter_key) {
			if (isset($_POST['filter_' . $filter_key]) && intval($_POST['filter_' . $filter_key]) !== -1) {
				$filters[$filter_key] = filter_input(INPUT_POST, 'filter_' . $filter_key);
			} else {
				$filters[$filter_key] = false;
			}
		}

		// Handle query variables depending on selected tab
		switch ($action) {
			case 'new_orders':
				$page_title = $page_params['strings'][$action];
				$args = array(
					'post_manifest' => false,
				);
				break;
			case 'completed_orders':
				$page_title = $page_params['strings'][$action];
				$args = array(
					'post_manifest' => true,
					// Latest manifest at the top
					'meta_query' => array(
						'relation' => 'OR',
						array(
							'key' => $configs['meta_keys']['manifest_date'],
						),
						array( // Compatible for older
							'key' => $configs['meta_keys']['manifest_date_old'],
						),
					),
					'orderby' => 'meta_value',
					'order' => 'DESC'
				);
				break;
			case 'all_orders':
			default:
				$action = 'all_orders';
				$page_title = $page_params['strings']['all_orders'];
				$args = array();
				break;
		}

		foreach ($filters as $key => $filter) {
			if ($filter) {
				switch ($key) {
					case 'status':
						$args = array_merge(
							$args,
							array('status' => $filter)
						);
						break;
					case 'barcode':
						$args = array_merge(
							$args,
							array('post_barcode' => $filter)
						);
						break;
					case 'customer':
						$args = array_merge(
							$args,
							array('post_customer' => $filter)
						);
						break;
				}
			}
		}
		// Date filter is a special case
		if ($filters['start_date'] || $filters['end_date']) {
			$args = array_merge(
				$args,
				array('post_manifest_date' => array($filters['start_date'], $filters['end_date']))
			);
		}

		// Get orders with extra info about the results.
		$shipping_methods = array('post');
		foreach ($configs['method_params'] as $method) {
			if (!$method['is_shipping_method']) continue;

			$shipping_methods[] = 'post_' . $method['key'];
		}
		$args = array_merge(
			$args,
			array(
				'shipping_method' => $shipping_methods,
				'paginate' => true,
				'limit' => $page_params['per_page'],
				'paged' => $paged,
			)
		);


		// Searching by ID takes priority
		$single_order = false;
		if ($filters['id']) {
			$single_order = wc_get_order($filters['id']);
			if ($single_order) {
				$orders = array($single_order); // Table printer expects array
				$paged = 1;
			}
		}

		// If there is no search by ID use to custom query
		$results = false;
		if (!$single_order) {
			$results = wc_get_orders($args);
			$orders = [];
			foreach ($results->orders as $key => $result) {
				$shipping = $result->get_items('shipping');
				foreach ($shipping as $key => $shipp) {
					if ($shipp->get_data()['method_id'] == 'post' || $shipp->get_data()['method_id'] == 'post_c') {
						$orders[] = $result;
					};
				}
			}
		}

		$there_is_orders = ($single_order || ($results && $results->total > 0));

		// make pagination
		$page_links = false;
		if ($results) {
			$page_links = paginate_links(array(
				'base' => add_query_arg('paged', '%#%'),
				'format' => '?paged=%#%',
				'prev_text' => __('&laquo;', 'text-domain'),
				'next_text' => __('&raquo;', 'text-domain'),
				'total' => $results->max_num_pages,
				'current' => $paged,
				'type' => 'plain'
			));
		}

		$order_statuses = wc_get_order_statuses();

		return array(
			'orders' => $orders,
			'statuses' => $order_statuses,
			'paged' => $paged,
			'action' => $action,
			'links' => $page_links,
			'is_orders' => $there_is_orders,
			'filters' => $filters,
		);
	}
}
