<?php
class Post_Order {
	public static function load_admin_scripts($hook) {
		global $post;

		if ($hook == 'post-new.php' || $hook == 'post.php') {
			if ($post->post_type === 'shop_order') {
				wp_enqueue_script('post_admin_order', plugins_url('/assets/js/post_admin_order.js', Post_Core::$main_file_path), array('jquery'), POST_VERSION);
			}
		}
	}

	public static function after_rate_description($method, $index) {
		if (is_cart()) return; // Exit on cart page

		$customer = WC()->session->get('customer');
		if (!isset($customer['country'])) {
			return;
		}

		$shipping_settings = Post_Core::get_settings();
		if (!isset($shipping_settings['prices_' . $customer['country']])) {
			return;
		}

		$rate_settings = json_decode($shipping_settings['prices_' . $customer['country']]);
		$shipping_methods = Post_Core::get_configs('method_params');

		foreach ($shipping_methods as $ship_method => $ship_method_values) {
			if (!$ship_method_values['is_shipping_method']) continue;

			if (!empty($rate_settings->{$ship_method_values['key'] . "_description"}) && $method->id === 'post_' . $ship_method_values['key']) {
				echo '<span class="post-shipping-description">' . $rate_settings->{$ship_method_values['key'] . "_description"} . '</span>';
			}
		}
	}

	public static function after_rate_terminals($method) {
		$customer = WC()->session->get('customer');
		$country = "ALL";
		if (isset($customer['shipping_country'])) {
			$country = $customer['shipping_country'];
		} elseif (isset($customer['country'])) {
			$country = $customer['country'];
		}
		$termnal_id = WC()->session->get('post_terminal_id');

		$selected_shipping_method = WC()->session->get('chosen_shipping_methods');
		if (empty($selected_shipping_method)) {
			$selected_shipping_method = array();
		}
		if (!is_array($selected_shipping_method)) {
			$selected_shipping_method = array($selected_shipping_method);
		}

		if ($method->id == "post_pt" && in_array("post_pt", $selected_shipping_method)) {
			echo Post_Terminals::get_terminals_options($termnal_id, $country);
		}
		if ($method->id == "post_ck" && in_array("post_ck", $selected_shipping_method)) {
			echo Post_Terminals::get_terminals_options($termnal_id, $country, 'circle');
		}
		if ($method->id == "post_po" && in_array("post_po", $selected_shipping_method)) {
			echo Post_Terminals::get_terminals_options($termnal_id, $country, 'post');
		}
	}

	/**
	 * Restrict Post Shipping methods if cart products has restricted categories
	 */
	public static function restrict_shipping_methods_by_cats($rates) {
		global $woocommerce;
		$configs = Post_Core::get_configs();
		$settings = get_option($configs['settings_key']);
		$cart_categories_ids = array();

		foreach ($woocommerce->cart->get_cart() as $cart_item) {
			$cats = get_the_terms($cart_item['product_id'], 'product_cat');
			if (empty($cats)) {
				continue;
			}
			foreach ($cats as $cat) {
				$cart_categories_ids[] = $cat->term_id;
				if ($cat->parent != 0) {
					$cart_categories_ids[] = $cat->parent;
				}
			}
		}
		$cart_categories_ids = array_unique($cart_categories_ids);

		$restricted_categories = $settings['restricted_categories'] ?? array();
		if (!is_array($restricted_categories)) {
			$restricted_categories = array($restricted_categories);
		}

		if(!empty($restricted_categories)) {
			foreach ($cart_categories_ids as $cart_product_categories_id) {
				if (in_array($cart_product_categories_id, $restricted_categories)) {
					foreach ($configs['method_params'] as $ship_method => $ship_method_values) {
						if (!$ship_method_values['is_shipping_method']) continue;
						unset($rates['post_' . $ship_method_values['key']]);
					}
					break;
				}
			}
		}

		return $rates;
	}

	public static function add_terminal_id_to_order($order_id) {
		$configs = Post_Core::get_configs();

		if (isset($_POST['post_terminal']) && $order_id) {
			update_post_meta($order_id, $configs['meta_keys']['terminal_id'], $_POST['post_terminal']);
		}

		if (isset($_POST['shipping_method']) && is_array($_POST['shipping_method'])) {
			foreach ($_POST['shipping_method'] as $ship_method) {
				foreach ($configs['method_params'] as $method_name => $method_values) {
					if (!$method_values['is_shipping_method']) continue;
					if ($ship_method == "post_" . $method_values['key']) {
						update_post_meta($order_id, $configs['meta_keys']['method'], $ship_method);
						break;
					}
				}
			}
		}
	}

	public static function validate_order($posted) {
		$packages = WC()->shipping->get_packages();
		$chosen_methods = WC()->session->get('chosen_shipping_methods');
		if (is_array($chosen_methods) && in_array('post', $chosen_methods)) {
			foreach ($packages as $i => $package) {
				if ($chosen_methods[$i] != 'post') {
					continue;
				}

				$shipping_settings = Post_Core::get_settings();
				$weightLimit = (int) $shipping_settings['weight'];
				$weight = 0;
				foreach ($package['contents'] as $item_id => $values) {
					$_product = $values['data'];
					$weight = $weight + $_product->get_weight() * $values['quantity'];
				}

				$weight = wc_get_weight($weight, 'kg');
				if ($weight > $weightLimit) {
					$message = sprintf(__('Sorry, %d kg exceeds the maximum weight of %d kg for %s', 'post'), $weight, $weightLimit, __('Post shipping', 'post'));
					$messageType = "error";
					if (!wc_has_notice($message, $messageType)) {
						wc_add_notice($message, $messageType);
					}
				}
			}
		}
	}

	public static function show_selected_terminal($order) {
		$configs_methods = Post_Core::get_configs('method_params');
		$send_method = self::get_post_method($order);

		foreach ($configs_methods as $method_key => $method_values) {
			if (!$method_values['is_shipping_method']) continue;
			if ($send_method != 'post_' . $method_values['key']) continue;

			if ($method_values['key'] == 'pt' || $method_values['key'] == 'po'  || $method_values['key'] == 'ck') {
				echo '<p>Post ' . strtolower($method_values['title']) . ": " . Post_Terminals::get_terminal_address($order) . '</p>';
			}
		}
	}

	/**
	 * Add custom order meta data to make it accessible in Order preview template
	 */
	public static function admin_order_add_custom_meta_data($data, $order) {
		$configs = Post_Core::get_configs();
		$send_method = self::get_post_method($order);

		foreach ($configs['method_params'] as $method_key => $method_values) {
			if (!$method_values['is_shipping_method']) continue;
			if ($send_method != 'post_' . $method_values['key']) continue;

			if ($method_values['key'] == 'pt' || $method_values['key'] == 'po') {
				$data['shipping_via'] = 'Post ' . strtolower($method_values['title']) . ": " . Post_Terminals::get_terminal_address($order);
			}
		}

		if ($send_method) {
			$barcode = $order->get_meta($configs['meta_keys']['barcode']);
			$data['post_barcode'] = $barcode;
		}

		return $data;
	}

	public static function bulk_actions($actions) {
		$actions['post_labels'] = __('Print Post labels', 'post');
		$actions['post_manifest'] = __('Print Post manifest', 'post');
		return $actions;
	}

	public static function handle_bulk_actions($redirect_to, $action, $ids) {
		if ($action == "post_labels") {
			$post_labels = new Post_Labels();
			$post_labels->print_labels($ids);
			return 0;
		}

		if ($action == "post_manifest") {
			$wc_shipping = new WC_Shipping();
			$post = new Post_Shipping_Method();
			$post->printBulkManifests($ids);
			return 0;
		}

		return $redirect_to;
	}

	public static function post_label_actions() {
		$regenerate = false;
		if (isset($_REQUEST['process']) && $_REQUEST['process'] === 'regenerate') {
			$regenerate = true;
		}
		if (!is_array($_REQUEST['post'])) {
			$_REQUEST['post'] = array($_REQUEST['post']);
		}

		$post_labels = new Post_Labels();
		$post_labels->print_labels($_REQUEST['post'], true, $regenerate);
	}

	public static function post_manifest_actions() {
		$wc_shipping = new WC_Shipping();
		$post = new Post_Shipping_Method();
		$post->printBulkManifests($_REQUEST['post']);
	}

	public static function generate_label() {
		if (current_user_can('edit_shop_orders') && check_admin_referer('woocommerce-mark-order-status')) {
			$post_labels = new Post_Labels();
			$post_labels->print_labels(array($_GET['order_id']), false);
		}
		wp_safe_redirect(wp_get_referer() ? wp_get_referer() : admin_url('edit.php?post_type=shop_order'));
		exit;
	}

	/**
	 * Add weight to order
	 */
	public static function add_cart_weight($order_id) {
		global $woocommerce;
		$weight = $woocommerce->cart->cart_contents_weight;
		update_post_meta($order_id, '_cart_weight', $weight);
	}

	/**
	 * Display and edit Post info on the order edit page
	 */
	public static function admin_order_display($order, $print_barcode = true, $admin_panel = true) {
		$configs = Post_Core::get_configs();
		$send_method = self::get_post_method($order);
		$post_labels = new Post_Labels();
		$is_post = false;
		foreach ($configs['method_params'] as $ship_method => $ship_values) {
			if (!$ship_values['is_shipping_method']) continue;

			if ($send_method == 'post_' . $ship_values['key']) {
				$is_post = true;
			}
		}

		if (!$is_post && $send_method !== 'post_c') return;
		global $post_type;
		$only_in_order = true;
		if ('shop_order' != $post_type) {
			$only_in_order = false;
		}

		if ($only_in_order) {
			echo '<br class="clear"/>';
			echo '<hr style="margin-top:20px;">';
			echo '<h4>' . __('Post shipping', 'post') . '</h4>';
		}

		echo '<div class="address">';
		foreach ($configs['method_params'] as $ship_method => $ship_values) {
			if (!$ship_values['is_shipping_method']) continue;
			if ($send_method != 'post_' . $ship_values['key']) continue;

			$field_value = $order->get_formatted_shipping_address();
			if ($ship_values['key'] == 'pt' || $ship_values['key'] == 'po' || $ship_values['key'] == 'ck') {
				$field_value = Post_Terminals::get_terminal_address($order);
			}

			echo '<p><strong class="title">' . $ship_values['title'] . ':</strong> ' . $field_value . '</p>';
		}
		if ($send_method == 'post_c') {
			$field_value = $order->get_formatted_shipping_address();
			echo '<p><strong class="title">' . __('ExpressPasts courier', 'post') . ':</strong> ' . $field_value . '</p>';
		}

		if ($print_barcode) {
			$barcode = $order->get_meta('_post_barcode') ?? '';
			echo $barcode;
		}
		echo '</div>';

		if (!$only_in_order) {
			return;
		}

		echo '<div class="edit_address">';
		if(in_array($send_method, ['post_pt','post_po','post_ck'])) {
			switch ($send_method) {
				case 'post_pt':
					$values = array(
						'terminal_key' => 'terminal',
						'change_title' => __('Change parcel terminal', 'post'),
					);
					break;
				case 'post_po':
					$values = array(
						'terminal_key' => 'post',
						'change_title' => __('Change post office', 'post'),
					);
					break;
				case 'post_ck':
					$values = array(
						'terminal_key' => 'circle',
						'change_title' => __('Change CircleK station', 'post'),
					);
					break;
			}

			$all_terminals = Post_Terminals::get_terminals_list($values['terminal_key']);
			$selected_terminal = get_post_meta($order->get_id(), $configs['meta_keys']['terminal_id'], true);
			$order_country = $order->get_shipping_country();

			echo '<p class="form-field-wide">';
			echo '<label for="post_terminal">' . $values['change_title'] . '</label>';
			echo '<input type="hidden" id="post-order-country" value="' . $order_country . '">';
			echo '<select id="post_terminal" class="select short" name="post_terminal_id">';
			echo '<option>-</option>';
			foreach ($all_terminals as $country => $country_terminals) {
				foreach ($country_terminals as $county => $terminals) {
					echo '<optgroup data-country="' . $country . '" label="' . $county . '">';
					foreach ($terminals as $terminal_id => $terminal_name) {
						$selected = (($terminal_id == $selected_terminal) && ($country == $order_country)) ? 'selected' : '';
						echo '<option value="' . $terminal_id . '" ' . $selected . '>' . $terminal_name . '</option>';
					}
					echo '</optgroup>';
				}
			}
			echo '</select>';
			echo '</p>';
		} else {
			echo __('The delivery address is changed in the fields above', 'post');
		}

		echo '</div>';
		echo '<hr style="margin-top:20px;">';
	}

	public static function admin_order_save($post_id) {
		global $post_type;
		if ('shop_order' != $post_type) {
			return $post_id;
		}

		$configs = Post_Core::get_configs();

		if (isset($_POST['post_terminal_id'])) {
			update_post_meta($post_id, $configs['meta_keys']['terminal_id'], wc_clean($_POST['post_terminal_id']));
		}

		return $post_id;
	}

	public static function checkout_validate_terminal() {
		$messages = array(
			'pt' => __('Please select parcel terminal.', 'post'),
			'po' => __('Please select post office.', 'post'),
			'ck' => __('Please select CircleK station.', 'post'),
		);

		foreach ($messages as $key => $message) {
			if (isset($_POST['shipping_method']) && in_array('post_' . $key, $_POST['shipping_method'])) {
				if (empty($_POST['post_terminal'])) {
					wc_add_notice($message, 'error');
				}
			}
		}
	}

	private static function get_post_method($order) {
		$configs_meta = Post_Core::get_configs('meta_keys');
		$wc_order = wc_get_order((int) $order->get_id());
		$send_method = "";
		foreach ($wc_order->get_items('shipping') as $item_id => $shipping_item_obj) {
			$send_method = $shipping_item_obj->get_method_id();
		}
		if ($send_method == 'post') {
			return get_post_meta($order->get_id(), $configs_meta['method'], true);
		} else {
			return $send_method;
		}
	}
}
