<?php
if (!class_exists('Post_Shipping_Method')) {
	class Post_Shipping_Method extends WC_Shipping_Method {
		/**
		 * Constructor for your shipping class
		 *
		 * @access public
		 * @return void
		 */
		public $errors = array();

		private $post_api;
		private $post_configs;
		private $shipping_sets;
		private $methods_asociations;
		private $destinations_countries = array();
		private $post_settings;
		public function __construct() {
			$this->id = 'post';
			$this->method_title = __('Expresspasts shipping', 'post');
			$this->method_description = __('Shipping methods for Expresspasts', 'post');

			$this->post_api = new Post_Api();
			$this->post_configs = Post_Core::get_configs();
			$this->post_settings = Post_Core::get_settings();
			$this->methods_asociations = Post_Helper::get_methods_asociations();

			// Destination countries
			foreach ($this->post_configs['shipping_params'] as $ship_params) {
				foreach ($ship_params['shipping_sets'] as $country => $set) {
					if ($country === 'call') continue;
					if (!isset($this->destinations_countries[$country])) {
						$country_name = $country;
						if (isset($this->post_configs['shipping_params'][$country]['title'])) {
							$country_name = $this->post_configs['shipping_params'][$country]['title'];
						}
						$this->destinations_countries[$country] = $country_name;
					}
				}
			}

			// Availability, Countries and other required Woocommerce functions
			$this->availability = 'including';
			$this->countries = array_keys($this->destinations_countries);

			$this->init();

			$this->enabled = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';
		}

		private function convert_method_name_to_short($method_name, $reverse = false) {
			foreach ($this->methods_asociations as $key => $value) {
				if (!$reverse) {
					if ($method_name === $value) {
						return $key;
					}
				} else {
					if ($method_name === $key) {
						return $value;
					}
				}
			}

			return $method_name;
		}

		/**
		 * Init your settings
		 *
		 * @access public
		 * @return void
		 */
		function init() {

			// Load the settings API


			$this->init_form_fields();
			$this->init_settings();

			//$this->title = $this->get_option('title');

			// Save settings in admin if you have any defined
			add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
			//$this->update_locations_file();
		}

		public function update_locations_file() {
			$locations = $this->post_configs['locations'];
			$fp = fopen(POST_DIR . '/' . "locations.json", "w");
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, $locations['source_url']);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_HEADER, false);
			curl_setopt($curl, CURLOPT_FILE, $fp);
			curl_setopt($curl, CURLOPT_TIMEOUT, 60);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			$data = curl_exec($curl);
			curl_close($curl);
			fclose($fp);
		}

		/**
		 * Load settings form
		 */
		function admin_options() {
?>
			<h2><?php echo $this->method_title; ?></h2>
			<p><?php echo $this->method_description; ?></p>
			<table class="form-table post-settings">
				<?php $this->generate_settings_html(); ?>
			</table>
			<?php
		}
		/**
		 * Define settings field for this shipping
		 * @return void
		 */
		function init_form_fields() {
			$countries_options = array();
			foreach ($this->post_configs['shipping_params'] as $country_code => $ship_params) {
				$countries_options[$country_code] = $country_code . ' - ' . $ship_params['title'];
			}

			$countries_post = @file_get_contents('https://ftp.webdev.lv/pasts/countries.json');
			$post_countries = json_decode($countries_post, true);
			$options_countries = array();
			if (!empty($post_countries)) {
				//Language code based on admin language
				$current_language = get_bloginfo('language');
				switch (mb_strtolower($current_language)) {
					case 'en-gb':
					case 'en':
					case 'en-en':
						$lang = 'en';
						break;
					case 'lv-lv':
					case 'lv':
						$lang = 'lv';
						break;
					case 'ru-ru':
					case 'ru':
						$lang = 'ru';
						break;
					default:
						$lang = 'en';
				}

				foreach ($post_countries as $result) {
					if (isset($result['Country'])) {
						$options_countries[$result['Country']['id']] = $result['Country']['title_' . $lang] . ' (' . $result['Country']['iso2'] . ')';
					}
				}
			}

			$fields = array(
				'enabled' => array(
					'title' => __('Enable', 'post'),
					'type' => 'checkbox',
					'description' => sprintf(__('Disable this setting to turn off all %s methods, except courier.', 'post'), $this->method_title),
					//'desc_tip' => true,
					'default' => 'yes',
				),
				'api_user' => array(
					'title' => __('API user', 'post'),
					'type' => 'text',
					'description' => __('Please contact Post for API access codes.', 'post'),
				),

				'hr_shop' => array(
					'type' => 'hr',
					'title' => __('Sender information', 'post'),
				),
				'company' => array(
					'title' => __('Company name', 'post'),
					'type' => 'text',
				),
				'bank_account' => array(
					'title' => __('Bank account', 'post'),
					'type' => 'text',
				),
				'shop_name' => array(
					'title' => __('Shop name', 'post'),
					'type' => 'text',
				),
				'shop_city' => array(
					'title' => __('Shop city', 'post'),
					'type' => 'text',
				),
				'shop_address' => array(
					'title' => __('Shop address', 'post'),
					'type' => 'text',
				),
				'shop_postcode' => array(
					'title' => __('Shop postcode', 'post'),
					'type' => 'text',
					'description' => sprintf(__('Example for Latvia: %1$s. Example for other countries: %2$s.', 'post'), '<code>LV-0123</code>', '<code>01234</code>'),
				),
				'shop_phone' => array(
					'title' => __('Shop phone number', 'post'),
					'type' => 'text',
					'description' => __('Required for calling courier', 'post'),
				),
				'shop_email' => array(
					'title' => __('Shop email', 'post'),
					'type' => 'text',
				),
				'pick_up_start' => array(
					'title' => __('Pick up time start', 'post'),
					'type' => 'text',
					'placeholder' => '08:00',
					'description' => sprintf(__('Allowed formats: %1$s. Default time is %2$s, if incorrect value is entered or field is empty.', 'post'), '<i>07:00, 7:00, 7</i>', '08:00'),
				),
				'pick_up_end' => array(
					'title' => __('Pick up time end', 'post'),
					'type' => 'text',
					'placeholder' => '17:00',
					'description' => sprintf(__('Allowed formats: %1$s. Default time is %2$s, if incorrect value is entered or field is empty.', 'post'), '<i>09:00, 9:00, 9</i>', '17:00'),
				),
				/*
        'multiparcels' => array(
          'title' => __('Multiparcels', 'post'),
          'type' => 'checkbox',
          'description' => __('Send from store type.', 'post'),
        ),
		*/
				'hs_code' => array(
					'title' => __('HS code', 'post'),
					'type' => 'text'
				),
				'origin_country_id' => array(
					'title' => __('Country of Origin', 'post'),
					'type' => 'select',
					'options' => $options_countries,
					'class' => 'checkout-style pickup-point'
				),
			);
			$fields['hr_prices'] = array(
				'type' => 'hr',
				'title' => __('Delivery countries and prices', 'post'),
			);
			foreach ($this->destinations_countries as $country_code => $country_name) {
				$fields['prices_' . $country_code] = array(
					'type' => 'prices_box',
					'lang' => $country_code,
				);
			}
			$fields['hr_settings'] = array(
				'type' => 'hr',
				'title' => __('Shipping methods settings', 'post'),
			);
			foreach ($this->post_configs['method_params'] as $ship_method => $ship_method_values) {

				$field_key = 'weight_' . $ship_method_values['key'];
				if ($ship_method_values['key'] === 'pt') {
					$field_key = 'weight';
				}

				$fields[$field_key] = array(
					'title' => sprintf(__('Max cart weight (%1$s) for %2$s', 'post'), 'kg', strtolower($ship_method_values['title'])),
					'type' => 'number',
					'custom_attributes' => array(
						'step' => 0.001,
						'min' => 0
					),
					'description' => sprintf(__('Maximum allowed all cart products weight for %s.', 'post'), strtolower($ship_method_values['title'])),
					'default' => $ship_method_values['weight']['default'],
					'class' => 'post_' . $ship_method,
				);
			}
			$fields['size_pt'] = array(
				'title' => sprintf(__('Max cart size (%s) for terminal', 'post'), get_option('woocommerce_dimension_unit')),
				'type' => 'dimensions',
				'description' => __('Maximum cart size for parcel terminals. Leave all empty to disable.', 'post') . '<br/>' . __('Preliminary cart size is calculated by trying to fit all products by taking their dimensions (boxes) indicated in their settings.', 'post'),
				'class' => 'post_terminal'
			);
			/*$fields['size_c'] = array(
        'title' => sprintf(__('Max size (%s) for courier', 'post'),get_option('woocommerce_dimension_unit')),
        'type' => 'dimensions',
        'description' => __('Maximum product size for courier. Leave all empty to disable.', 'post') . '<br/>' . __('If the length, width or height of at least one product exceeds the specified values, then it will not be possible to select the courier delivery method for the whole cart.', 'post')
      );*/
			$fields['restricted_categories'] = array(
				'title' => __('Disable for specific categories', 'post'),
				'type' => 'multiselect',
				'class' => 'wc-enhanced-select',
				'description' => __('Select categories for which you want to disable the Post method.', 'post'),
				'options' => $this->post_get_categories(),
				//'desc_tip' => true,
				'required' => false,
				'custom_attributes' => array(
					'data-placeholder' => __('Select Categories', 'post'),
					'data-name' => 'restricted_categories'
				),
			);
			$fields['auto_select'] = array(
				'title' => __('Automatic terminal selection', 'post'),
				'type' => 'checkbox',
				'description' => __('Automatically select terminal by postcode.', 'post'),
				'default' => 'yes',
				'class' => 'post_terminal'
			);
			$fields['hr_design'] = array(
				'type' => 'hr',
				'title' => __('Design', 'post'),
			);
			/*
      $fields['show_map'] = array(
        'title' => __('Map', 'post'),
        'type' => 'checkbox',
        'description' => __('Show map of terminals.', 'post'),
        'default' => 'yes',
        'class' => 'post_terminal'
      );
	  */
			$fields['custom_label'] = array(
				'title' => __('Custom label names', 'post'),
				'type' => 'label_name',
				'description' => __('Use custom shipping method name.', 'post') . ' ' . __('Values is not translatable.', 'post'),
			);
			$fields['position'] = array(
				'title' => __('Positions', 'post'),
				'type' => 'position',
				'description' => __('Position of each Expresspasts shipping method in shipping methods list on Checkout page.', 'post') . '<br/>' . __('Leave empty to not change position. A higher number means a lower position (1 - top of the list).', 'post') . '<br/>' . __('NOTE', 'post') . ': ' . __('Positioning may be affected by other plugins or functions used in the theme.', 'post'),
			);

			$fields['hr_manifest'] = array(
				'type' => 'hr',
				'title' => __('Manifest', 'post'),
			);
			$fields['manifest_enable'] = array(
				'title' => __('Enable manifest print', 'post'),
				'type' => 'checkbox',
				'description' => __('Allow print manifest. Disable this option will hide table manifest column and manifest generation buttons.', 'post'),
				'default' => 'yes',
			);

			$fields['hr_debug'] = array(
				'type' => 'hr',
				'title' => __('Debug', 'post'),
			);
			$fields['debug_mode'] = array(
				'title' => __('Enable debug mode', 'post'),
				'type' => 'checkbox',
				'description' => __('Enable request and response logging.', 'post') . ' ' . sprintf(__('Log files are stored for %d days.', 'post'), $this->post_configs['debug']['delete_after']),
				'default' => ''
			);
			$fields['debugview_request'] = array(
				'type' => 'debug_window',
				'files' => Post_Debug::get_all_files(),
				'title' => __('Logged data', 'post'),
				'class' => 'post_debug'
			);
			$fields['hr_end'] = array(
				'type' => 'hr',
			);
			$this->form_fields = $fields;
		}

		public function generate_hr_html($key, $value) {
			$class = (isset($value['class'])) ? $value['class'] : '';
			$title = '';
			if (!empty($value['title'])) {
				if (!empty($class)) {
					$class .= ' ';
				}
				$class .= 'have_title';
				$title = '<span>' . $value['title'] . '</span>';
			}

			$html = '<tr valign="top"><td colspan="2" class="section_title"><hr class="' . $class . '">' . $title . '</td></tr>';

			return $html;
		}

		public function generate_empty_html($key, $value) {
			$class = (isset($value['class'])) ? $value['class'] : '';

			$html = '<tr valign="top"><td colspan="2" class="' . $class . '"></td></tr>';

			return $html;
		}

		public function generate_string_html($key, $value) {
			$class = (isset($value['class'])) ? $value['class'] : '';

			$html = '<tr valign="top">';
			$html .= '<th scope="row" class="titledesc"><label>' . $value['title'] . '</label></th>';
			$html .= '<td class="forminp"><fieldset><p class="description">' . $value['text'] . '</p></fieldset></td>';
			$html .= '</tr>';

			return $html;
		}

		public function generate_prices_box_html($key, $value) {
			$box_key = $this->get_field_key($key);
			$html = '';
			if ($value['lang'] == 'LV' || $value['lang'] == 'LT') {
				$flag_img_url = POST_URL . 'assets/img/flags/' . strtolower($value['lang']) . '.png';
				if (isset($this->post_configs['shipping_params'][$value['lang']])) {
					$shipping_methods = $this->post_configs['shipping_params'][$value['lang']]['methods'];
					$shipping_keys = array();
					foreach ($shipping_methods as $ship_method) {
						$shipping_keys[] = $this->convert_method_name_to_short($ship_method);
					}
				} else {
					$shipping_keys = array_keys($this->methods_asociations);
				}
				$fields = array();
				foreach ($shipping_keys as $ship_key) {
					$fields[$ship_key . '_enable'] = $ship_key . '_enable_' . $value['lang'];
					$fields[$ship_key . '_price_type'] = $ship_key . '_price_type_' . $value['lang'];
					$fields[$ship_key . '_price_single'] = $ship_key . '_price_' . $value['lang'];
					$fields[$ship_key . '_price_by_weight'] = $ship_key . '_price_by_weight_' . $value['lang'];
					$fields[$ship_key . '_price_by_amount'] = $ship_key . '_price_by_amount_' . $value['lang'];
					$fields[$ship_key . '_enable_free_from'] = $ship_key . '_price_' . $value['lang'] . '_enFree';
					$fields[$ship_key . '_free_from'] = $ship_key . '_price_' . $value['lang'] . '_FREE';
					$fields[$ship_key . '_enable_coupon'] = $ship_key . '_price_' . $value['lang'] . '_enCoupon';
					$fields[$ship_key . '_coupon'] = $ship_key . '_price_' . $value['lang'] . '_coupon';
					$fields[$ship_key . '_description'] = $ship_key . '_description_' . $value['lang'];
				}
				/* START Fields only for parcel terminal */
				$fields['pt_price_by_boxsize'] = 'pt_price_by_box_' . $value['lang'];
				/* END Fields only for parcel terminal */
				$saved_values = json_decode($this->get_option($key));
				$values = array();
				foreach ($fields as $id => $field) {
					$cur_value = (isset($saved_values->{$id})) ? $saved_values->{$id} : '';
					/* -Compatibility with old data- */
					$old_value = $this->get_option($field);
					$is_old = false;
					if ($cur_value === '' && (!empty($old_value) || $old_value === 0 || $old_value === '0')) {
						$cur_value = $old_value;
						$is_old = true;
					}
					/* -End of Compatibility with old data- */
					$values[$id] = array(
						'id' => $field,
						'key' => $this->get_field_key($field),
						'value' => $cur_value,
						'is_old' => $is_old,
					);
				}
			} else {
				$flag_img_url = '';
				if (isset($this->post_configs['shipping_params'][''])) {
					$shipping_methods = $this->post_configs['shipping_params']['']['methods'];
					$shipping_keys = array();
					foreach ($shipping_methods as $ship_method) {
						$shipping_keys[] = $this->convert_method_name_to_short($ship_method);
					}
				} else {
					$shipping_keys = array_keys($this->methods_asociations);
				}
				$fields = array();
				foreach ($shipping_keys as $ship_key) {
					$fields[$ship_key . '_enable'] = $ship_key . '_enable';
					$fields[$ship_key . '_price_type'] = $ship_key . '_price_type';
					$fields[$ship_key . '_price_single'] = $ship_key . '_price';
					$fields[$ship_key . '_price_by_weight'] = $ship_key . '_price_by_weight';
					$fields[$ship_key . '_price_by_amount'] = $ship_key . '_price_by_amount';
					$fields[$ship_key . '_enable_free_from'] = $ship_key . '_price_enFree';
					$fields[$ship_key . '_free_from'] = $ship_key . '_price_FREE';
					$fields[$ship_key . '_enable_coupon'] = $ship_key . '_price_enCoupon';
					$fields[$ship_key . '_coupon'] = $ship_key . '_price_coupon';
					$fields[$ship_key . '_description'] = $ship_key . '_description';
				}
				/* START Fields only for parcel terminal */
				$fields['pt_price_by_boxsize'] = 'pt_price_by_box';
				/* END Fields only for parcel terminal */
				$saved_values = json_decode($this->get_option($key));
				$values = array();
				foreach ($fields as $id => $field) {
					$cur_value = (isset($saved_values->{$id})) ? $saved_values->{$id} : '';
					/* -Compatibility with old data- */
					$old_value = $this->get_option($field);
					$is_old = false;
					if ($cur_value === '' && (!empty($old_value) || $old_value === 0 || $old_value === '0')) {
						$cur_value = $old_value;
						$is_old = true;
					}
					/* -End of Compatibility with old data- */
					$values[$id] = array(
						'id' => $field,
						'key' => $this->get_field_key($field),
						'value' => $cur_value,
						'is_old' => $is_old,
					);
				}
			}
			$args = array(
				'posts_per_page'   => -1,
				'orderby'          => 'title',
				'order'            => 'asc',
				'post_type'        => 'shop_coupon',
				'post_status'      => 'publish',
			);
			$coupons = get_posts($args);

			ob_start();
			if ($value['lang'] == 'EU' || $value['lang'] == 'LV' || $value['lang'] == 'LT') { ?>

				<tr class="row-prices" valign="top">
					<td colspan="2">
						<div class="prices_box">
							<div class="pb-lang">
								<?php if ($value['lang'] !== 'ALL') { ?>
									<img src="<?php echo $flag_img_url; ?>" alt="[<?php echo $value['lang']; ?>]">
								<?php  }; ?>
								<span><?php echo $value['lang'] . ' ' . __('prices', 'post'); ?></span>
							</div>
							<div class="pb-content">
								<?php foreach ($shipping_keys as $ship_key) : ?>
									<?php if (isset($values[$ship_key . '_enable'])) : ?>

										<?php
										$params = array(
											'lang' => $value['lang'],
											'box_key' => $box_key,
											'enable' => array(
												'id' => $values[$ship_key . '_enable']['key'],
												'name' => $ship_key . '_enable',
												'checked' => ($values[$ship_key . '_enable']['value']) ? 'checked' : '',
												'class' => $ship_key . '_enable',
											),
											'prices' => array(
												'type' => (isset($values[$ship_key . '_price_type'])) ? $values[$ship_key . '_price_type'] : false,
												'type_name' => $ship_key . '_price_type',
												'single' => (isset($values[$ship_key . '_price_single'])) ? $values[$ship_key . '_price_single'] : false,
												'single_name' => $ship_key . '_price_single',
												'weight' => (isset($values[$ship_key . '_price_by_weight'])) ? $values[$ship_key . '_price_by_weight'] : false,
												'weight_name' => $ship_key . '_price_by_weight',
												'amount' => (isset($values[$ship_key . '_price_by_amount'])) ? $values[$ship_key . '_price_by_amount'] : false,
												'amount_name' => $ship_key . '_price_by_amount',
												'free_enable' => (isset($values[$ship_key . '_enable_free_from'])) ? $values[$ship_key . '_enable_free_from'] : false,
												'free_enable_name' => $ship_key . '_enable_free_from',
												'free_enable_class' => $ship_key . '_enable_free',
												'free' => (isset($values[$ship_key . '_free_from'])) ? $values[$ship_key . '_free_from'] : false,
												'free_name' => $ship_key . '_free_from',
												'coupon' => (isset($values[$ship_key . '_coupon'])) ? $values[$ship_key . '_coupon'] : false,
												'coupon_name' => $ship_key . '_coupon',
												'coupon_enable' => (isset($values[$ship_key . '_enable_coupon'])) ? $values[$ship_key . '_enable_coupon'] : false,
												'coupon_enable_name' => $ship_key . '_enable_coupon',
												'coupon_enable_class' => $ship_key . '_enable_coupon',
											),
											'data' => array(
												'coupons' => $coupons,
											),
											'other' => array(
												'desc' => (isset($values[$ship_key . '_description'])) ? $values[$ship_key . '_description'] : false,
												'desc_name' => $ship_key . '_description',
											),
										);
										foreach ($this->post_configs['method_params'] as $method_name => $method_values) {
											if ($ship_key === $method_values['key']) {
												$params['type'] = $method_name;
												$params['title'] = $method_values['title'];
												$params['enable']['title'] = sprintf(__('Enable %s', 'post'), strtolower($method_values['title']));
												break;
											}
										}
										if ($ship_key === 'pt') {
											$params['prices']['boxsize'] = (isset($values[$ship_key . '_price_by_boxsize'])) ? $values[$ship_key . '_price_by_boxsize'] : false;
											$params['prices']['boxsize_name'] = $ship_key . '_price_by_boxsize';
										}

										echo $this->post_build_prices_block($params);
										?>
									<?php endif; ?>
								<?php endforeach; ?>
							</div>
						</div>
					</td>
				</tr>
			<?php };
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}

		private function post_build_prices_block($params) {
			$params = array(
				'lang' => (isset($params['lang'])) ? $params['lang'] : '',
				'type' => (isset($params['type'])) ? $params['type'] : '',
				'title' => (isset($params['title'])) ? $params['title'] : __('Shipping', 'post'),
				'box_key' => (isset($params['box_key'])) ? $params['box_key'] : '',
				'enable' => array(
					'title' => (isset($params['enable']['title'])) ? $params['enable']['title'] : __('Enable', 'post'),
					'id' => (isset($params['enable']['id'])) ? $params['enable']['id'] : '',
					'name' => (isset($params['enable']['name'])) ? $params['enable']['name'] : '',
					'checked' => (isset($params['enable']['checked'])) ? $params['enable']['checked'] : '',
					'class' => (isset($params['enable']['class'])) ? $params['enable']['class'] : '',
				),
				'prices' => array(
					'type' => (isset($params['prices']['type'])) ? $params['prices']['type'] : false,
					'type_name' => (isset($params['prices']['type_name'])) ? $params['prices']['type_name'] : '',
					'single' => (isset($params['prices']['single'])) ? $params['prices']['single'] : false,
					'single_name' => (isset($params['prices']['single_name'])) ? $params['prices']['single_name'] : '',
					'single_title' => (isset($params['prices']['single_title'])) ? $params['prices']['single_title'] : __('Price', 'post'),
					'weight' => (isset($params['prices']['weight'])) ? $params['prices']['weight'] : false,
					'weight_name' => (isset($params['prices']['weight_name'])) ? $params['prices']['weight_name'] : '',
					'weight_title' => (isset($params['prices']['weight_title'])) ? $params['prices']['weight_title'] : __('Weight', 'post'),
					'amount' => (isset($params['prices']['amount'])) ? $params['prices']['amount'] : false,
					'amount_name' => (isset($params['prices']['amount_name'])) ? $params['prices']['amount_name'] : '',
					'amount_title' => (isset($params['prices']['amount_title'])) ? $params['prices']['amount_title'] : __('Cart amount', 'post'),
					'boxsize' => (isset($params['prices']['boxsize'])) ? $params['prices']['boxsize'] : false,
					'boxsize_name' => (isset($params['prices']['boxsize_name'])) ? $params['prices']['boxsize_name'] : '',
					'boxsize_title' => (isset($params['prices']['boxsize_title'])) ? $params['prices']['boxsize_title'] : __('Box size', 'post'),
					'free_enable' => (isset($params['prices']['free_enable'])) ? $params['prices']['free_enable'] : false,
					'free_enable_name' => (isset($params['prices']['free_enable_name'])) ? $params['prices']['free_enable_name'] : '',
					'free_enable_class' => (isset($params['prices']['free_enable_class'])) ? $params['prices']['free_enable_class'] : '',
					'free' => (isset($params['prices']['free'])) ? $params['prices']['free'] : false,
					'free_name' => (isset($params['prices']['free_name'])) ? $params['prices']['free_name'] : '',
					'free_title' => (isset($params['prices']['free_title'])) ? $params['prices']['free_title'] : __('Free from', 'post'),
					'coupon' => (isset($params['prices']['coupon'])) ? $params['prices']['coupon'] : false,
					'coupon_name' => (isset($params['prices']['coupon_name'])) ? $params['prices']['coupon_name'] : '',
					'coupon_title' => (isset($params['prices']['coupon_title'])) ? $params['prices']['coupon_title'] : __('Free with coupon', 'post'),
					'coupon_enable' => (isset($params['prices']['coupon_enable'])) ? $params['prices']['coupon_enable'] : false,
					'coupon_enable_name' => (isset($params['prices']['coupon_enable_name'])) ? $params['prices']['coupon_enable_name'] : '',
					'coupon_enable_class' => (isset($params['prices']['coupon_enable_class'])) ? $params['prices']['coupon_enable_class'] : '',
				),
				'data' => array(
					'coupons' => (isset($params['data']['coupons'])) ? $params['data']['coupons'] : array(),
				),
				'other' => array(
					'desc' => (isset($params['other']['desc'])) ? $params['other']['desc'] : false,
					'desc_name' => (isset($params['other']['desc_name'])) ? $params['other']['desc_name'] : '',
					'desc_title' => (isset($params['other']['desc_title'])) ? $params['other']['desc_title'] : __('Description', 'post'),
				),
			);

			if (empty($params['type']) || empty($params['box_key'])) {
				return '';
			}

			ob_start();
			if ($params['lang'] == 'EU' && ($params['type'] == 'circle' || $params['type'] == 'post' || $params['type'] == 'terminal')) {
			} else { ?>
				<div class="block-prices <?php echo $params['type']; ?>">
					<div class="sec-title">
						<?php
						/* -Compatibility with old data- */
						if ($params['prices']['single']['is_old'] && isset($params['prices']['single']['value']) && $params['prices']['single']['value'] !== '') {
							$params['enable']['checked'] = 'checked';
						}
						/* -End of Compatibility with old data- */

						$html_params = array(
							'label' => $params['title'],
							'title' => $params['enable']['title'],
							'id' => $params['enable']['id'],
							'name' => $params['box_key'] . '[' . $params['enable']['name'] . ']',
							'class' => $params['enable']['class'],
							'checked' => ($params['enable']['checked'] === 'checked') ? true : false,
						);

						echo Post_Admin_Html::buildSwitcher($html_params);
						?>
					</div>
					<div class="sec-prices">
						<?php if ($params['prices']['type'] !== false) : ?>
							<?php
							$html_params = array(
								'field_id' => $params['prices']['type']['key'],
								'field_name' => $params['box_key'] . '[' . $params['prices']['type_name'] . ']',
								'field_value' => $params['prices']['type']['value'],
							);
							if ($params['prices']['boxsize'] !== false) {
								$html_params['add_select_options'] = array(
									'boxsize' => __('By box size', 'post'),
								);
							}
							echo Post_Admin_Html::buildPriceType($html_params);
							?>
						<?php endif; ?>
						<?php if ($params['prices']['single'] !== false) : ?>
							<div class="prices-single">
								<?php
								$field_value = $params['prices']['single']['value'];
								if (empty($field_value) && $field_value !== 0 && $field_value !== '0') {
									$field_value = 2;
								}
								$html_params = array(
									'label' => $params['prices']['single_title'] . ':',
									'id' => $params['prices']['single']['key'],
									'type' => 'number',
									'name' => $params['box_key'] . '[' . $params['prices']['single_name'] . ']',
									'value' => $field_value,
									'step' => 0.01,
									'min' => 0,
								);
								echo Post_Admin_Html::buildSimpleField($html_params);
								?>
							</div>
						<?php endif; ?>
						<?php if ($params['prices']['weight'] !== false) : ?>
							<?php
							$html_params = array(
								'type' => 'weight',
								'field_id' => $params['prices']['weight']['key'],
								'field_name' => $params['box_key'] . '[' . $params['prices']['weight_name'] . ']',
								'values' => $params['prices']['weight']['value'],
								'c1_title' => $params['prices']['weight_title'] . ' (' . get_option('woocommerce_weight_unit') . ')',
								'c1_step' => 0.001,
							);
							echo Post_Admin_Html::buildPricesTable($html_params);
							?>
						<?php endif; ?>
						<?php if ($params['prices']['amount'] !== false) : ?>
							<?php
							$html_params = array(
								'type' => 'amount',
								'field_id' => $params['prices']['amount']['key'],
								'field_name' => $params['box_key'] . '[' . $params['prices']['amount_name'] . ']',
								'values' => $params['prices']['amount']['value'],
								'c1_title' => $params['prices']['amount_title'],
								'c1_step' => 0.01,
							);
							echo Post_Admin_Html::buildPricesTable($html_params);
							?>
						<?php endif; ?>
						<?php if ($params['prices']['boxsize'] !== false) : ?>
							<?php
							$method_params = $this->post_configs['method_params'];
							$box_sizes = array();
							if (isset($method_params[$params['type']]['sizes'])) {
								foreach ($method_params[$params['type']]['sizes'] as $key => $sizes) {
									if ($key !== 'min') {
										$box_sizes[] = $key;
									}
								}
							}
							if (empty($params['prices']['boxsize']['value'])) {
								$default_values = array();
								for ($i = 0; $i < count($box_sizes); $i++) {
									$default_values[] = (object) array(
										'value' => $box_sizes[$i],
										'price' => 2,
									);
								}
								$params['prices']['boxsize']['value'] = (object) $default_values;
							} else {
								$i = 0;
								foreach ($params['prices']['boxsize']['value'] as $value) {
									if (isset($box_sizes[$i])) {
										$value->value = $box_sizes[$i];
									}
									$i++;
								}
							}
							$box_titles = $method_params[$params['type']]['titles'];
							foreach ($box_titles as $key => $title) {
								$h = $method_params[$params['type']]['sizes'][$key][0];
								$w = $method_params[$params['type']]['sizes'][$key][1];
								$l = $method_params[$params['type']]['sizes'][$key][2];
								$text = sprintf(__('Max %s cm', 'post'), $h . '×' . $w . '×' . $l);
								$box_titles[$key] = $title . '<br/><small>' . $text . '</small>';
							}
							$html_params = array(
								'type' => 'boxsize',
								'field_id' => $params['prices']['boxsize']['key'],
								'field_name' => $params['box_key'] . '[' . $params['prices']['boxsize_name'] . ']',
								'values' => $params['prices']['boxsize']['value'],
								'c1_title' => $params['prices']['boxsize_title'],
								'allow_add' => false,
								'c1_text' => $box_titles,
								'desc' => __('NOTE', 'post') . ': ' . __('If at least one item in the cart does not have the specified size, then this shipping method will not be displayed', 'post'),
							);
							echo Post_Admin_Html::buildPricesTable($html_params);
							?>
						<?php endif; ?>
						<?php if ($params['prices']['free'] !== false) : ?>
							<div class="prices-free">
								<?php
								$field_checked = ($params['prices']['free_enable']['value']) ? 'checked' : '';
								/* -Compatibility with old data- */
								if ($params['prices']['free']['is_old'] && isset($params['prices']['free']['value']) && $params['prices']['free']['value'] !== '') {
									$field_checked = 'checked';
								}
								/* -End of Compatibility with old data- */
								$html_params = array(
									'label' => $params['prices']['free_title'] . ':',
									'label_position' => 'after',
									'id' => $params['prices']['free_enable']['key'],
									'class' => $params['prices']['free_enable_class'],
									'name' => $params['box_key'] . '[' . $params['prices']['free_enable_name'] . ']',
									'checked' => ($field_checked === 'checked') ? true : false,
									'value' => 1,
								);
								echo Post_Admin_Html::buildCheckbox($html_params);

								$field_value = $params['prices']['free']['value'];
								if (empty($field_value) && $field_value != 0) {
									$field_value = 100;
								}
								$html_params = array(
									'id' => $params['prices']['free']['key'],
									'type' => 'number',
									'name' => $params['box_key'] . '[' . $params['prices']['free_name'] . ']',
									'value' => $field_value,
									'step' => 0.01,
									'min' => 0,
									'class' => 'input-text regular-input price_free',
								);
								echo ' ' . Post_Admin_Html::buildSimpleField($html_params);
								?>
							</div>
						<?php endif; ?>
						<?php if ($params['prices']['coupon'] !== false) : ?>
							<div class="prices-coupon">
								<?php
								$field_checked = ($params['prices']['coupon']['value']) ? 'checked' : '';
								$html_params = array(
									'label' => $params['prices']['coupon_title'] . ':',
									'label_position' => 'after',
									'id' => $params['prices']['coupon_enable']['key'],
									'class' => $params['prices']['coupon_enable_class'],
									'name' => $params['box_key'] . '[' . $params['prices']['coupon_enable_name'] . ']',
									'checked' => ($field_checked === 'checked') ? true : false,
									'value' => 1,
								);
								echo Post_Admin_Html::buildCheckbox($html_params);

								$options = array();
								foreach ($params['data']['coupons'] as $coupon) {
									$options[] = array(
										'value' => strtolower($coupon->post_title),
										'title' => $coupon->post_title,
									);
								}
								$selected = (empty($params['prices']['coupon']['value'])) ? 'selected' : '';
								$html_params = array(
									'name' => $params['box_key'] . '[' . $params['prices']['coupon_name'] . ']',
									'id' => $params['prices']['coupon']['key'],
									'class' => 'price_coupon',
									'options' => $options,
									'selected' => $params['prices']['coupon']['value'],
								);
								echo ' ' . Post_Admin_Html::buildSelectField($html_params);
								?>
							</div>
						<?php endif; ?>
					</div>
					<div class="sec-other">
						<?php if ($params['other']['desc'] !== false) : ?>
							<div class="other-description">
								<?php
								$html_params = array(
									'label' => $params['other']['desc_title'] . ':',
									'id' => $params['other']['desc']['key'],
									'name' => $params['box_key'] . '[' . $params['other']['desc_name'] . ']',
									'value' => $params['other']['desc']['value'],
								);
								echo Post_Admin_Html::buildTextareaField($html_params);
								?>
							</div>
						<?php endif; ?>
					</div>
				</div>
			<?php }
			$html = ob_get_contents();
			ob_end_clean();

			return $html;
		}

		public function validate_prices_box_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		public function generate_dimensions_html($key, $value) {
			$field_key = $this->get_field_key($key);
			$field_class = (isset($value['class'])) ? $value['class'] : '';

			if ($this->get_option($key) !== '') {
				$dim_values = $this->get_option($key);
				if (is_string($dim_values)) {
					$dim_values = json_decode($this->get_option($key), true);
				}
			} else {
				$dim_values = array();
			}

			ob_start();
			?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<label><?php echo esc_html($value['title']); ?></label>
				</th>
				<td class="forminp">
					<fieldset class="field-dimensions <?php echo $field_class; ?>">
						<input type="number" value="<?php echo $dim_values[0] ?? ''; ?>" id="<?php echo esc_html($field_key); ?>_0" name="<?php echo esc_html($field_key); ?>[0]" min="0.001" step="0.001" placeholder="<?php echo __('Length', 'post'); ?>">
						<span>x</span>
						<input type="number" value="<?php echo $dim_values[1] ?? ''; ?>" id="<?php echo esc_html($field_key); ?>_1" name="<?php echo esc_html($field_key); ?>[1]" min="0.001" step="0.001" placeholder="<?php echo __('Width', 'post'); ?>">
						<span>x</span>
						<input type="number" value="<?php echo $dim_values[2] ?? ''; ?>" id="<?php echo esc_html($field_key); ?>_2" name="<?php echo esc_html($field_key); ?>[2]" min="0.001" step="0.001" placeholder="<?php echo __('Height', 'post'); ?>">
						<span><?php echo get_option('woocommerce_dimension_unit'); ?></span>
						<?php if (!empty($value['description'])) : ?>
							<p class="description"><?php echo __($value['description']); ?></p>
						<?php endif; ?>
					</fieldset>
				</td>
			</tr>
		<?php
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}
		public function validate_dimensions_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		public function generate_label_name_html($key, $value) {
			$field_key = $this->get_field_key($key);
			$field_class = (isset($value['class'])) ? $value['class'] : '';
			$field_values = array();

			if ($this->get_option($key) !== '') {
				$field_values = $this->get_option($key);
				if (is_string($field_values)) {
					$field_values = json_decode($this->get_option($key), true);
				}
			}

			$avalable_methods = Post_Shipmethod_Helper::get_available_shipping_methods($this->post_configs);
			$splited_methods = array_chunk($avalable_methods, 3, true);

			ob_start();
		?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<label><?php echo esc_html($value['title']); ?></label>
				</th>
				<td class="forminp">
					<fieldset class="field-custom_label <?php echo $field_class; ?>">
						<table>
							<?php foreach ($splited_methods as $methods_row) : ?>
								<tr>
									<?php foreach ($methods_row as $method_key => $method_values) : ?>
										<th><?php echo $method_values['title']; ?></th>
									<?php endforeach; ?>
								</tr>
								<tr>
									<?php foreach ($methods_row as $method_key => $method_values) : ?>
										<?php $current_value = (isset($field_values[$method_values['key']])) ? $field_values[$method_values['key']] : ""; ?>
										<td>
											<input type="text" name="<?php echo esc_html($field_key); ?>[<?php echo esc_html($method_values['key']); ?>]" value="<?php echo esc_html($current_value); ?>">
										</td>
									<?php endforeach; ?>
								</tr>
							<?php endforeach; ?>
						</table>
						<p class="description"><?php echo $value['description']; ?></p>
					</fieldset>
				</td>
			</tr>
		<?php
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}
		public function validate_label_name_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		public function generate_position_html($key, $value) {

			$field_key = $this->get_field_key($key);
			$field_class = (isset($value['class'])) ? $value['class'] : '';
			$field_values = array();

			if ($this->get_option($key) !== '') {
				$field_values = $this->get_option($key);
				if (is_string($field_values)) {
					$field_values = json_decode($this->get_option($key), true);
				}
			}

			$avalable_methods = Post_Shipmethod_Helper::get_available_shipping_methods($this->post_configs);
			$splited_methods = array_chunk($avalable_methods, 5, true);

			ob_start();

		?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<label><?php echo esc_html($value['title']); ?></label>
				</th>
				<td class="forminp">
					<fieldset class="field-position <?php echo $field_class; ?>">
						<table>
							<?php foreach ($splited_methods as $methods_row) : ?>
								<tr>
									<?php foreach ($methods_row as $method_key => $method_values) : ?>
										<th><?php echo $method_values['title']; ?></th>
									<?php endforeach; ?>
									<th><?php echo __('Post courier', 'post'); ?></th>
								</tr>
								<tr>
									<?php foreach ($methods_row as $method_key => $method_values) : ?>
										<?php $current_value = (isset($field_values[$method_values['key']])) ? $field_values[$method_values['key']] : ""; ?>
										<td>
											<input type="number" name="<?php echo esc_html($field_key); ?>[<?php echo esc_html($method_values['key']); ?>]" value="<?php echo esc_html($current_value); ?>" min="0" max="90" step="1">
										</td>
									<?php endforeach; ?>
									<?php $current_value = (isset($field_values['courier'])) ? $field_values['c'] : ""; ?>
									<td>
										<input type="text" name="woocommerce_post_position[c]" value="<?php echo esc_html($current_value); ?>">
									</td>
								</tr>
							<?php endforeach; ?>
						</table>
						<p class="description"><?php echo $value['description']; ?></p>
					</fieldset>
				</td>
			</tr>
		<?php
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}
		public function validate_position_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		/**
		 * Get categories for "restricted_categories" field
		 */
		public function post_get_categories() {
			$cats = $this->get_categories_hierarchy();
			$result = [];

			foreach ($cats as $item) {
				$this->create_categories_list('', $item, $result);
			}

			return $result;
		}

		/**
		 * Makes a list of categories to select from in settings page. array(lowest cat id => full cat path name)
		 */
		private function create_categories_list($prefix, $data, &$results) {
			if ($prefix) {
				$prefix = $prefix . ' &gt; ';
				$results[$data->term_id] = $prefix . $data->name;
			}
			if (!$data->children) {
				$results[$data->term_id] = $prefix . $data->name;

				return true;
			}

			foreach ($data->children as $child) {
				$this->create_categories_list($prefix . $data->name, $child, $results);
			}
		}

		private function get_categories_hierarchy($parent = 0) {
			$taxonomy = 'product_cat';
			$orderby = 'name';
			$hide_empty = 0;

			$args = array(
				'taxonomy'   => $taxonomy,
				'parent'     => $parent,
				'orderby'    => $orderby,
				'hide_empty' => $hide_empty,
			);

			$cats = get_categories($args);
			$children = array();

			foreach ($cats as $cat) {
				$cat->children = $this->get_categories_hierarchy($cat->term_id);
				$children[$cat->term_id] = $cat;
			}

			return $children;
		}

		public function generate_debug_window_html($key, $value) {
			$field_class = (isset($value['class'])) ? $value['class'] : '';
			$files = (isset($value['files'])) ? $value['files'] : array();
			$files_dir = Post_Debug::$_debug_dir;

			ob_start();
		?>
			<tr class="post-debugview" valign="top">
				<th scope="row" class="titledesc"></th>
				<td class="forminp">
					<fieldset class="field-debug <?php echo $field_class; ?>">
						<span class="title"><?php echo esc_html($value['title']); ?></span>
						<?php if (empty($files)) : ?>
							<textarea readonly rows="2" style="width:100%">- <?php echo __('Debug files still not created', 'post'); ?> -</textarea>
						<?php else : ?>
							<?php foreach ($files as $file_data) : ?>
								<div class="debug-row">
									<?php
									$file_path = $files_dir . $file_data['name'];
									$file = fopen($file_path, 'r');
									if (filesize($file_path) > 0) {
										$file_content = fread($file, filesize($file_path));
									} else {
										$file_content = '- ' . __('File is empty', 'post') . ' -';
									}
									fclose($file);
									if (!empty($file_data['day'])) {
										$date = date("Y-m-d", strtotime($file_data['day']));
									} else {
										$date = __('Date unknown', 'post');
									}
									?>
									<span class="date"><?= esc_html($date); ?></span>
									<textarea readonly rows="11" style="width:100%;display:none;"><?php echo $file_content; ?></textarea>
								</div>
							<?php endforeach; ?>
						<?php endif; ?>
					</fieldset>
				</td>
			</tr>
<?php
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}

		/**
		 * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
		 *
		 * @access public
		 * @param mixed $package
		 * @return void
		 */
		public function calculate_shipping($package = array()) {
			$weight = 0;
			$cost = 0;
			$country = $package["destination"]["country"];

			global $woocommerce;
			$cart_amount = $woocommerce->cart->cart_contents_total + $woocommerce->cart->tax_total;

			$products_for_dim = array();
			foreach ($package['contents'] as $item_id => $values) {
				$_product = $values['data'];
				if ($_product->get_weight()) {
					$weight = $weight + $_product->get_weight() * $values['quantity'];
				}
				for ($i = 0; $i < $values['quantity']; $i++) {
					array_push($products_for_dim, $_product);
				}
			}

			$weight = wc_get_weight($weight, 'kg');

			$prices_key = (array_key_exists($country, $this->post_configs['shipping_params'])) ? 'prices_' . $country : 'prices_LT';
			$prices = (isset($this->settings[$prices_key])) ? json_decode($this->settings[$prices_key]) : array();

			$this->add_shipping_rate('pt', $products_for_dim, $weight, $country, $cart_amount, $prices, $package);
			//$this->add_shipping_rate('c', false, $weight, $country, $cart_amount, $prices, $package); //courier comes from Shipping Zones
			$this->add_shipping_rate('po', false, $weight, $country, $cart_amount, $prices, $package);
			$this->add_shipping_rate('ck', false, $weight, $country, $cart_amount, $prices, $package);
		}

		private function add_shipping_rate($rate_key, $products_for_dim, $weight, $country, $cart_amount, $prices, $package) {
			$method_params = Post_Shipmethod_Helper::get_current_method_params($this->post_configs['method_params'], $rate_key);

			$check_restrictions = Post_Shipmethod_Helper::check_restrictions($this->settings, $rate_key, $weight, $products_for_dim);

			if ($check_restrictions) {
				$show = true;
				$amount_data = Post_Shipmethod_Helper::get_amount($rate_key, $prices, $weight, $cart_amount);
				$amount = $amount_data['amount'];
				$meta_data = $amount_data['meta_data'];

				if (empty($amount)) {
					$show = false;
				}
				if (!isset($prices->{$rate_key . '_enable'})) {
					$show = false;
				}

				$amount = Post_Shipmethod_Helper::check_amount_free($rate_key, $prices, $amount, $cart_amount);
				$amount = Post_Shipmethod_Helper::check_coupon($rate_key, $prices, $amount, $package['applied_coupons']);

				$rate_name = $method_params['title'];
				if (!empty($this->settings['custom_label'])) {
					$custom_labels = json_decode($this->settings['custom_label']);
					$rate_name = (!empty($custom_labels->{$rate_key})) ? $custom_labels->{$rate_key} : $rate_name;
				}

				$rate = array(
					'id' => 'post_' . $rate_key,
					'label' => $rate_name,
					'cost' => $amount,
					'meta_data' => $meta_data,
				);

				if (!Post_Shipmethod_Helper::is_rate_allowed($rate_key, $country)) {
					$show = false;
				}

				if ($show) {
					$this->add_rate($rate);
				}
			}
		}

		function printBulkManifests($orderIds = array()) {
			$url = 'https://express.pasts.lv/parcelsDocumentsApi/manifest';
			$secret = $this->post_settings['api_user'];
			$labels = new Post_Labels;
			$error = $labels->create_parcel($orderIds, false); //Create parcels, if they don't exist

			if (empty($error)) {
				$barcodes = [];
				foreach ($orderIds as $orderId) {
					$barcode = get_post_meta($orderId, $this->post_configs['meta_keys']['barcode'], true);
					$barcodes = array_merge($barcodes, explode(',', $barcode));
				}

				$data = array(
					"secret" => $secret,
					"parcels" => $barcodes,
				);
				$response = $labels->send($url, $data);

				//Debug
				$data['url'] = $url;
				Post_Debug::debug_write($data, "MANIFEST REQUEST");
				//Post_Debug::debug_write($response, "MANIFEST RESPONSE");
				
				$data = json_decode($response, true);
				if (!is_array($response) && json_last_error() != 0) {
					$time = current_time('Y-m-d H:i:s');
					foreach ($orderIds as $orderId) {
						update_post_meta($orderId, '_post_manifest_date', $time);
					}

					$name = 'manifest_' . time() . '.pdf';
					ob_start();
					header('Content-Description: File Transfer');
					header('Content-type: application/pdf');
					header('Content-Disposition: attachment; filename="' . $name . '"');
					header("Expires: 0");
					header("Cache-Control: must-revalidate");
					header("Pragma: public");
					header('Content-Transfer-Encoding: binary');
					header('Content-Length: ' . strlen($response));
					ob_end_flush();

					echo $response;
				} else {
					$err_msg = sprintf(__('Can\'t generate %s. API response: ', 'post'), 'manifest');
					Post_Helper::add_msg($err_msg . json_encode($response, JSON_UNESCAPED_UNICODE), 'error');
					wp_safe_redirect(wp_get_referer());
				}
			} else {
				foreach ($error as $id => $err) {
					Post_Helper::add_msg('Order #' . $id . ' - ' . $err, 'error');
				}
				wp_safe_redirect(wp_get_referer());
			}
		}

		function get_terminal_address($terminal_id) {
			$terminals_json_file_dir = POST_DIR . '/' . "locations.json";
			$terminals_file = fopen($terminals_json_file_dir, "r");
			$terminals = fread($terminals_file, filesize($terminals_json_file_dir) + 10);
			fclose($terminals_file);
			$terminals = json_decode($terminals, true);
			$parcel_terminals = '';
			if (is_array($terminals) && $terminal_id) {
				foreach ($terminals as $terminal) {
					if ($terminal['code'] == $terminal_id) {
						return $terminal['title'] . ', ' . $terminal['region'] . ', ' . preg_replace("/[^a-zA-Z0-9]+/", "", $terminal['code']);
					}
				}
			}
			return $terminal_id . '';
		}
	}
}
