<?php
class Post_Terminals {
	public static function add_terminal_to_session() {
		if (isset($_POST['terminal_id']) && is_numeric($_POST['terminal_id'])) {
			WC()->session->set('post_terminal_id', $_POST['terminal_id']);
		}
		wp_die();
	}

	public static function get_terminals_list($get_list) {
		if ($get_list === 'post') {
			$terminals = self::read_terminals_file1();
		} elseif ($get_list === 'circle') {
			$terminals = self::read_terminals_file2();
			$terminals = array('lv' => $terminals);
		} else {
			$terminals = self::read_terminals_file();
		}

		$grouped_options = array();
		if (is_array($terminals)) {
			foreach ($terminals as $c=>$ct) {
				$country = mb_strtoupper($c);
				foreach($ct as $terminal) {
					if (isset($terminal['area'])) { // CircleK and LV post offices and terminals
						$nov = ($terminal['area'] !== 'NULL') ? $terminal['area'] : $terminal['city'];
					} elseif (!empty($terminal['city'])) { //LT terminals
						$nov = $terminal['city'];
					}else{ //LT post offices
						$nov = '';
					}

					$title = $terminal['title'] ?? $terminal['label'];
					$id = $terminal['id'] ?? $terminal['code'];

					$grouped_options[$country][$nov][$id] = $title;
				}
			}
		}

		$grouped_options = self::sort_terminals_list($grouped_options);
		return $grouped_options;
	}

	public static function get_terminals_options($selected = '', $country = "ALL", $get_list = 'terminals') {
		if ($get_list === 'post') {
			$terminals = self::read_terminals_file1();
			if ($country === 'LV') {
				$terminals = $terminals['lv'];
			}elseif ($country == 'LT') {
				$terminals = $terminals['lt'];
			}

			$list_options = array(
				'list' => 'post',
				'type' => 1,
				'txt_select' => __('Select post office', 'post'),
				'txt_show_map' => __('Show post offices map', 'post'),
			);
		}elseif ($get_list === 'circle') {
			$terminals = self::read_terminals_file2();
			$list_options = array(
				'list' => 'circle',
				'type' => '',
				'txt_select' => __('Select CircleK station', 'post'),
				'txt_show_map' => __('Show CircleK station map', 'post'),
			);
		}elseif ($get_list === 'terminals') {
			$terminals = self::read_terminals_file();
			if ($country === 'LV') {
				$terminals = $terminals['lv'];
			}elseif ($country == 'LT') {
				$terminals = $terminals['lt'];
			}

			$list_options = array(
				'list' => 'terminal',
				'type' => 0,
				'txt_select' => __('Select parcel terminal', 'post'),
				'txt_show_map' => __('Show parcel terminals map', 'post'),
			);
		}

		function cmp($a, $b) {
			if (!isset($a["city"]) && !isset($b["city"]) && !isset($a["area"]) && !isset($b["area"])) {
				return 0;
			}

			$city_a = !empty($a["city"]) ? $a["city"] : $a["area"];
			$city_b = !empty($b["city"]) ? $b["city"] : $b["area"];

			if ($city_a == $city_b) {
				return 0;
			}
			return ($city_a < $city_b) ? -1 : 1;
		}
		usort($terminals, 'cmp');

		$parcel_terminals = '';
		if (is_array($terminals)) {
			$grouped_options = array();
			foreach ($terminals as $terminal) {
				if (isset($terminal['city'])) {
					$code = $terminal['code'] ?? '';
					$pcode = $terminal['postal_code'] ?? '';
					$address = $terminal['address'] ?? '';
					if ((!str_contains($code, $country) && !str_contains($pcode, $country) && !str_contains($address, $country)) && $country != "ALL") continue;

					if (!isset($grouped_options[$terminal['city']])) $grouped_options[(string) $terminal['city']] = array();
					if (isset($terminal['additional_information'])) {
						$label = $terminal['address'] . '(' . $terminal['additional_information'] . ')';
					} else {
						$label = $terminal['title'] ?? $terminal['label'];
					}

					if (isset($terminal['id'])) {
						$id = $terminal['id'];
					} else {
						$id = $terminal['code'];
					}
					$grouped_options[(string) $terminal['city']][(string) $id] = $label;
				} else {
					if (!isset($terminal['title']) && !isset($terminal['additional_information'])) {
						$label = $terminal['label'];
					}
					if (isset($terminal['id'])) {
						$id = $terminal['id'];
					} else {
						$id = $terminal['code'];
					}

					$grouped_options[$id] = $label;
				}
			}
			$counter = 0;
			if (!empty($id)) {
				foreach ($grouped_options as $city => $locs) {
					if (is_array($locs)) {
						$parcel_terminals .= '<optgroup data-id = "' . $counter . '" label = "' . $city . '">';

						foreach ($locs as $key => $loc) {
							$parcel_terminals .= '<option value = "' . $key . '" ' . ($key == $selected ? 'selected' : '') . '>' . $loc . '</option>';
						}

						$parcel_terminals .= '</optgroup>';
					} else {
						$parcel_terminals .= '<option value = "' . $city . '" ' . ($city == $selected ? 'selected' : '') . '>' . $locs . '</option>';
					}
					$counter++;
				}
			}
		}
		$nonce = wp_create_nonce("post_terminals_json_nonce");
		$post_settings = get_option(Post_Core::get_configs('settings_key'));
		$parcel_terminals = '<option value = "">' . $list_options['txt_select'] . '</option>' . $parcel_terminals;
		$set_autoselect = (isset($post_settings['auto_select'])) ? $post_settings['auto_select'] : 'yes';

		$script = "<script style='display:none;'>
      var postTerminals = JSON.stringify(" . json_encode(self::get_terminals_for_map($country, $get_list)) . ");
    </script>";
		$script .= "<script style='display:none;'>
      var post_current_country = '" . $country . "';
      var postSettings = {
        auto_select:'" . $set_autoselect . "'
      };
      
      var post_type = '" . $get_list . "';
      var post_current_terminal = '" . $selected . "';
      jQuery('document').ready(function($){        
        $('.post_terminal').post();
        $(document).trigger('post.checkpostcode');
      });
      </script>";

		$button = '';

		return '<div class="terminal-container"><select class="post_terminal" name="post_terminal">' . $parcel_terminals . '</select>
      ' . $button . ' </div>' . $script;
	}

	public static function get_terminals_for_map($country = "LT", $get_list = 'terminal') {
		if ($country === 'LV') {
			$terminal = self::read_terminals_file();
			$terminals = $terminal['lv'];
		}
		if ($country == 'LT') {
			$terminal = self::read_terminals_file();
			$terminals = $terminal['lt'];
		}

		$terminalsList = array();
		if (is_array($terminals)) {
			if ($get_list === 'post') {
				if ($country === 'LV') {
					$terminal = self::read_terminals_file1();
					$terminals = $terminal['lv'];
				}
				if ($country == 'LT') {
					$terminal = self::read_terminals_file1();
					$terminals = $terminal['lt'];
				}
			};
			if ($get_list === 'circle') {
				$terminals = self::read_terminals_file2();
			};
			usort($terminals, 'cmp');

			foreach ($terminals as $terminal) {
				$postal_code = $terminal['postal_code'] ?? '';
				$code = $terminal['code'] ?? 0;
				if ($country == 'LT' && ($code > 1000  || strpos($postal_code, 'LT'))) {
					$title = $terminal['title'] ?? $terminal['label'];
					$lat = $terminal['lat'] ?? 1;
					$long = $terminal['long'] ?? 1;
					$zipcode = $terminal['code'] ?? $terminal['zipcode'];
					$id = $terminal['id'] ?? $terminal['code'];

					if (isset($terminal['area'])) {
						$area = $terminal['area'] !== 'NULL' ? $terminal['area'] : '';
					} else {
						$area = '';
					}

					if (isset($terminal['city'])) {
						$city = $terminal['city'] !== 'NULL' ? $terminal['city'] : '';
					} else {
						$city = '';
					}

					$terminalsList[] = [$title, $lat, $long,  $zipcode, $area, $city, '', $id];
				} else {
					$title = $terminal['title'] ?? $terminal['label'];
					if (!empty($terminal['business_place'])) {
						$title = $terminal['business_place'] . ' (' . $title . ')';
					}

					$lat = $terminal['lat'] ?? 1;
					$long = $terminal['long'] ?? 1;
					$zipcode = $terminal['zipcode'] ?? '';
					if (isset($terminal['area'])) {
						$area = $terminal['area'] !== 'NULL' ? $terminal['area'] : '';
					} else {
						$area = '';
					}

					if (isset($terminal['city'])) {
						$city = $terminal['city'] !== 'NULL' ? $terminal['city'] : '';
					} else {
						$city = '';
					}

					$id = $terminal['id'] ?? $terminal['code'];

					$terminalsList[] = [$title, $lat, $long, $zipcode, $area, $city, '', $id];
				}
			}
		}
		return $terminalsList;
	}

	public static function get_terminal_name($terminal_code, $shipping_method_id, $country) {
		$terminals = self::read_terminals_file();
		if ($shipping_method_id === 'post_pt') {
			if ($country == 'LV') {
				$terminals_file = fopen(POST_DIR . 'locations.json', "r");
				$terminals = fread($terminals_file, filesize(POST_DIR . 'locations.json') + 10);
				fclose($terminals_file);
				$terminals = json_decode($terminals, true);
			} else {
				$terminals_file_lt = fopen(POST_DIR . 'locations_lt.json', "r");
				$terminals_lt = fread($terminals_file_lt, filesize(POST_DIR . 'locations_lt.json') + 10);
				fclose($terminals_file_lt);
				$terminals = json_decode($terminals_lt, true);
			}
		};
		if ($shipping_method_id === 'post_po') {
			$terminals_file = fopen(POST_DIR . 'locations_office.json', "r");
			$terminals = fread($terminals_file, filesize(POST_DIR . 'locations_office.json') + 10);
			fclose($terminals_file);
			$terminals_file_lt = fopen(POST_DIR . 'locations_office_lt.json', "r");
			$terminals_lt = fread($terminals_file_lt, filesize(POST_DIR . 'locations_office_lt.json') + 10);
			fclose($terminals_file_lt);
			$terminals = array_merge(json_decode($terminals, true), json_decode($terminals_lt, true));
		};
		if ($shipping_method_id === 'post_ck') {
			$terminals = self::read_terminals_file2();
		};

		$parcel_terminals = '';
		if (is_array($terminals)) {
			foreach ($terminals as $terminal) {
				if (isset($terminal['id'])) {
					$id = $terminal['id'];
				} else {
					$id = $terminal['code'];
				}
				if ($id == $terminal_code) {
					$title = $terminal['title'] ?? $terminal['label'];
					if (!empty($terminal['business_place'])) {
						$title = $terminal['business_place'] . ' (' . $title . ')';
					}
					if (isset($terminal['city'])) {
						return $title . ', ' . $terminal['city'];
					} else {
						return $title;
					}
				}
			}
		}
		return false;
	}

	public static function get_terminal_address($order) {
		$configs_meta = Post_Core::get_configs('meta_keys');
		$shipping_method_id = get_post_meta($order->get_id(), '_post_method', true);
		$country =  $order->get_shipping_country();
		$terminal_id = get_post_meta($order->get_id(), $configs_meta['terminal_id'], true);
		$terminal_name = self::get_terminal_name($terminal_id, $shipping_method_id, $country);
		if (!$terminal_name) {
			$terminal_name  = __('Location not found!!!', 'post');
		}

		return $terminal_name;
	}

	public static function terminals_modal() {
		return '
    <div id="PostModal" class="modal">
        <div class="post-modal-content">
            <div class="post-modal-header">
            <span class="close" id="terminalsModal">&times;</span>
            <h5 id="post_modal_title" style="display: inline">' . __('Post parcel terminals', 'post') . '</h5>
            </div>
            <div class="post-modal-body" style="/*overflow: hidden;*/">
                <div id = "postMapContainer"></div>
                <div class="post-search-bar" >
                    <h4 id="post_modal_search" style="margin-top: 0px;">' . __('Parcel terminals addresses', 'post') . '</h4>
                    <div id="post-search">
                    <form>
                    <input type = "text" placeholder = "' . __('Enter postcode', 'post') . '"/>
                    <button type = "submit" id="map-search-button"></button>
                    </form>                    
                    <div class="post-autocomplete scrollbar" style = "display:none;"><ul></ul></div>
                    </div>
                    <div class = "post-back-to-list" style = "display:none;">' . __('Back', 'post') . '</div>
                    <div class="found_terminals1 scrollbar" id="style-8">
                      <ul>
                      
                      </ul>
                    </div>
                </div>
            </div>
        </div>
    </div>';
	}

	public static function check_terminals_json_file() {
		if (!file_exists(POST_DIR . 'locations.json')) {
			Post_Cronjob::generate_locations_file();
		}
	}

	public static function get_terminals_json() {
		if (!wp_verify_nonce($_REQUEST['nonce'], "post_terminals_json_nonce")) {
			exit("Not allowed");
		}

		$terminals_json = self::generate_terminals_json($_REQUEST['q'], $_REQUEST['country']);
		echo json_encode($terminals_json);

		die();
	}

	private static function read_terminals_file() {
		$terminals_file = fopen(POST_DIR . 'locations.json', "r");
		$terminals = fread($terminals_file, filesize(POST_DIR . 'locations.json') + 10);
		fclose($terminals_file);
		$terminals_file_lt = fopen(POST_DIR . 'locations_lt.json', "r");
		$terminals_lt = fread($terminals_file_lt, filesize(POST_DIR . 'locations_lt.json') + 10);
		fclose($terminals_file_lt);
		return array_merge(array('lv' => json_decode($terminals, true)), array('lt' => json_decode($terminals_lt, true)));
	}
	private static function read_terminals_file1() {
		$terminals_file = fopen(POST_DIR . 'locations_office.json', "r");
		$terminals = fread($terminals_file, filesize(POST_DIR . 'locations_office.json') + 10);
		fclose($terminals_file);
		$terminals_file_lt = fopen(POST_DIR . 'locations_office_lt.json', "r");
		$terminals_lt = fread($terminals_file_lt, filesize(POST_DIR . 'locations_office_lt.json') + 10);
		fclose($terminals_file_lt);
		return array_merge(array('lv' => json_decode($terminals, true)), array('lt' => json_decode($terminals_lt, true)));
	}
	private static function read_terminals_file2() {
		$terminals_file = fopen(POST_DIR . 'circle_locations.json', "r");
		$terminals = fread($terminals_file, filesize(POST_DIR . 'circle_locations.json') + 10);
		fclose($terminals_file);
		$terminals = json_decode($terminals, true);
		usort($terminals, function ($a, $b) {
			return $a['area'] <=> $b['area'];
		});
		return $terminals;
	}
	private static function generate_terminals_json($term = "", $country = "ALL", $get_list = 'terminal') {
		$c_p = false;
		if (strlen($term) >= 4 && strlen($term)) {
			$c_p = self::search_postcode($term, $country);
		}
		$terminals = self::read_terminals_file();
		$parcel_terminals = array();

		if (is_array($terminals)) {
			$grouped_options = array();
			$type = 0;
			if ($get_list === 'post' || $get_list === 'circle') $type = 1;
			foreach ($terminals as $terminal) {
				if ((!str_contains($terminal['code'], $country) && !str_contains($terminal['postal_code'], $country)) && $country != "ALL") continue;

				if (!isset($grouped_options[$terminal['region']])) $grouped_options[(string) $terminal['area']] = array();
				$grouped_options[(string) $terminal['region']][(string) $terminal['zipcode']] = $terminal;
			}
			$counter = 0;
			foreach ($grouped_options as $city => $locs) {
				$group = array("text" =>  $city, "distance" => 0, "children" => array());
				$group_distance = false;
				foreach ($locs as $key => $loc) {
					if ($term != "" && $c_p == false && stripos($loc['label'], $term) !== false) {
						$group['children'][] = array("id" => $key, "text" => $loc['label'], "distance" => 0);
					} elseif (is_array($c_p)) {
						$distance = self::calc_distance($c_p[0], $c_p[1], $loc['Y_COORDINATE'], $loc['X_COORDINATE']);
						$group['children'][] = array("id" => $key, "text" => $loc['label'], "distance" => $distance);
						if ($group_distance == false || $group_distance > $distance) {
							$group_distance = $distance;
						}
					} elseif ($term == "") {
						$group['children'][] = array("id" => $key, "text" => $loc['label'], "distance" => 0);
					}
				}
				$group['distance'] = $group_distance;
				if (count($group['children']) && $c_p == false) {
					$parcel_terminals[] = $group;
				} elseif (count($group['children'])) {
					$parcel_terminals = array_merge($parcel_terminals, $group['children']);
				}
				$counter++;
			}
		}
		if ($c_p != false) {
			usort($parcel_terminals, function ($a, $b) {
				return $b['distance'] > $a['distance'] ? -1 : 1;
			});
			return array_slice($parcel_terminals, 0, 8);
		}
		return $parcel_terminals;
	}

	private static function search_postcode($postcode, $country) {
		if ($postcode == "") return false;
		$postcode = urlencode($postcode);
		$data = file_get_contents("http://geocode.arcgis.com/arcgis/rest/services/World/GeocodeServer/findAddressCandidates?singleLine=" . $postcode . "," . $country . "&category=&outFields=Postal&maxLocations=1&forStorage=false&f=pjson");
		if ($data) {
			$data = json_decode($data);
		} else {
			return false;
		}
		if (isset($data->candidates) && count($data->candidates)) {
			if ($data->candidates[0]->score > 90) {
				return array($data->candidates[0]->location->y, $data->candidates[0]->location->x);
			}
		}
		return false;
	}

	private static function calc_distance($latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $earthRadius = 6371000) {
		// convert from degrees to radians
		$latFrom = deg2rad($latitudeFrom);
		$lonFrom = deg2rad($longitudeFrom);
		$latTo = deg2rad($latitudeTo);
		$lonTo = deg2rad($longitudeTo);

		$latDelta = $latTo - $latFrom;
		$lonDelta = $lonTo - $lonFrom;

		$angle = 2 * asin(sqrt(pow(sin($latDelta / 2), 2) +
			cos($latFrom) * cos($latTo) * pow(sin($lonDelta / 2), 2)));

		return round($angle * $earthRadius / 1000, 2);
	}

	private static function sort_terminals_list($list) {
		$sorted_list = array();
		foreach ($list as $key => $elem) {
			ksort($elem);
			$sorted_list[$key] = $elem;
		}

		return $sorted_list;
	}
}
