<?php
/**
 * Plugin Name: MansPasts shipping
 * Description: Official MansPasts shipping plugin for WooCommerce
 * Author: MansPasts
 * Version: 1.11.1
 * Domain Path: /languages
 * Text Domain: manspasts
 * Requires at least: 5.1
 * Tested up to: 6.0
 * WC requires at least: 3.0.0
 * WC tested up to: 6.6.1
 * Requires PHP: 7.2
 */

if (!defined('WPINC')) {
  die; 
}

define('MANSPASTS_VERSION', '1.11.1');
define('MANSPASTS_DIR', plugin_dir_path(__FILE__));
define('MANSPASTS_URL', plugin_dir_url(__FILE__));
define('MANSPASTS_BASENAME', plugin_basename(__FILE__));
define('MANSPASTS_CUSTOM_CHANGES', array()); //If plugin have custom changes, add changes descriptions to this array

function manspasts_configs($section_name = false) {
  $params = array();
  /*
   * Params for every shipping method
   *
   * Required values:
   * key (string) - Method key
   * title (string) - Method title
   * is_shipping_method (boolean) - If this method is shipping method. Using to exclude methods which using only in "send off" parameter.
   */
  $params['method_params'] = array(

     'post' => array(
      'key' => 'po',
      'title' => __('Post Offices', 'post'),
      'is_shipping_method' => true,
      'description' => __('Activate this service, when you want to send parcels within Latvia and Lithuania.', 'post'),
      'weight' => array(
        'default' => 100,
      ),
    )
  );

  /*
   * Variables, which using to replace some value in string. Using like {variable_key}.
   */
  $params['text_variables'] = array(
    'order_number' => __('Order number', 'manspasts'),
  );

  $params['meta_keys'] = array(
    'manifest_date_old' => '_manifest_generation_date',
  );

  /*
   * Debug params
   *
   * delete_after (integer) - The number of days after which to delete old debug files
   */
  $params['debug'] = array(
    'delete_after' => 30,
  );

  /*
   * Returns
   */
  if (!empty($section_name) && isset($params[$section_name])) {
    return $params[$section_name];
  }

  return $params;
}

/**
 * Plugin loading
 */
require_once MANSPASTS_DIR . 'core/class-core.php';
$manspasts_core = new Manspasts_Core();
$manspasts_core->init();
add_filter( 'woocommerce_available_payment_gateways', 'manspasts_gateway_disable_for_shipping_rate' );
  
function manspasts_gateway_disable_for_shipping_rate( $available_gateways ) {
  if ( !is_admin() ) {
    if(WC()->session){
		$chosen_method = WC()->session->get( 'chosen_shipping_methods' );
		if (strpos($chosen_method[0], 'manspasts_') !== false) {
			unset( $available_gateways['cod'] );
		 }
    }
  }
  return $available_gateways;
}
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    function Manspastscourier_shipping_method() {
        if (!class_exists('Manspastscourier_Shipping_Method')) {

            class Manspastscourier_Shipping_Method extends WC_Shipping_Method {
                public function __construct( $instance_id = 0 ) {
                    $this->id = 'manspasts_mm';
                    $this->instance_id          = absint( $instance_id );
                    $this->method_title         = __('Manspasts Courier', 'manspasts');
                    $this->supports             = array(
                        'shipping-zones',
                        'instance-settings',
                        'instance-settings-modal',
                    );

                    $this->init();
                }

                public function init() {
                    // Load the settings.
                    $this->init_form_fields();
                    $this->init_settings();

                    // Define user set variables.
                    $this->title    = isset($this->settings['title']) ? $this->settings['title'] : __('Manspasts Courier', 'manspasts');

                    add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
                }

                /**
                 * Init form fields.
                 */
                public function init_form_fields() {
                    $this->instance_form_fields = array(
                        'title'      => array(
                          'title'         => __( 'Title', 'manspasts' ),
                          'type'          => 'text',
                          'description'   => __( 'This controls the title which the user sees during checkout.', 'manspasts' ),
                          'default'       => __('Manspasts Courier', 'manspasts'),
                          'desc_tip'      => true,
                        ),
                        'cost' => array(
                          'title'         => __( 'Cost', 'manspasts' ),
                          'type'          => 'text',
                          'default'          => 0,
						  'description' 	=> __('Enter a single number or weight:price pairs, each pair seperated by comma', 'manspasts'),
						  'desc_tip'      => true,
                        ),
						'free_from' => array(
							'title'         => __('Free from:', 'manspasts'),
							'type'          => 'number',
							'custom_attributes' => array(
								'step' => 0.01,
								'min' => 0
							),
						),
                        'item_type-postage_type' => array(
                          'title'         => __( 'Item type - Postage type', 'manspasts' ),
                          'type'          => 'select',
                          'options'       => array('Parcel_Registered' => 'Parcel - Registered',
                            'Letter_Registered' => 'Letter - Registered',
                            'Letter_Ordinary' => 'Letter - Ordinary',
                            'Letter_Tracked' => 'Letter - Tracked'
                          ),
                          'default'       => 0,
                        ),
                    );
                }

                /**
                 * Get setting form fields for instances of this shipping method within zones.
                 *
                 * @return array
                 */
                public function get_instance_form_fields() {
                    return parent::get_instance_form_fields();
                }

                /**
                 * Always return shipping method is available
                 *
                 * @param array $package Shipping package.
                 * @return bool
                 */
                public function is_available( $package ) {
                    $is_available = true;
                    return apply_filters( 'woocommerce_shipping_' . $this->id . '_is_available', $is_available, $package, $this );
                }

                /**
                 * Free shipping rate applied for this method.
                 *
                 * @uses WC_Shipping_Method::add_rate()
                 *
                 * @param array $package Shipping package.
                 */
                public function calculate_shipping( $package = array() ) {
                    $instance_settings =  $this->instance_settings;
					$cost = $instance_settings['cost'];
					global $woocommerce;

					//Calculate cost based on weight
					$cart_weight = $woocommerce->cart->cart_contents_weight;
					$rates = explode(',', $cost);

					foreach ($rates as $rate) {
						$data = explode(':', $rate);
						if(isset($data[1])) {
							if ($data[0] >= $cart_weight) {
								$cost = $data[1];
								break;
							}
						}else{
							$cost = $data[0];
						}
					}

					//Free shipping from amount
					if($instance_settings['free_from'] > 0) {
						$cart_amount = $woocommerce->cart->cart_contents_total + $woocommerce->cart->tax_total;
						if($cart_amount >= $instance_settings['free_from']) {
							$cost = 0;
						}
					}

                    $this->add_rate(
                        array(
                            'label'   => !empty($instance_settings['title']) ? $instance_settings['title'] : $this->title,
                            'cost'    => $cost,
                            'taxes'   => false,
                            'package' => $package,
                        )
                    );
                }
            }
        }
    }
    add_action('woocommerce_shipping_init', 'Manspastscourier_Shipping_Method');

    function add_manspastscourier_shipping_method($methods) {
    $methods['manspasts_mm'] = 'manspastscourier_shipping_method'; 
    return $methods;
    }
    add_filter('woocommerce_shipping_methods', 'add_manspastscourier_shipping_method');

}