<?php
class Manspasts_Api
{
  private $manspasts_settings;
  private $manspasts_configs;

  public function __construct()
  {
    $this->manspasts_configs = Manspasts_Core::get_configs();
    $this->manspasts_settings = get_option($this->manspasts_configs['settings_key']);
  }

  public function get_client_data($order, $object = true)
  {
    $data = array(
      'name' => $this->clean($order->get_shipping_first_name()),
      'surname' => $this->clean($order->get_shipping_last_name()),
      'address_1' => $this->clean($order->get_shipping_address_1()),
      'postcode' => $this->clean($order->get_shipping_postcode()),
      'city' => $this->clean($order->get_shipping_city()),
      'country' => $this->clean($order->get_shipping_country()),
      'email' => $this->clean($order->get_billing_email()),
      'phone' => get_post_meta($order->get_id(), '_shipping_phone', true),
    );

    if ( empty($data['postcode']) && empty($data['city']) && empty($data['address_1']) && empty($data['country']) ) {
      $data['postcode'] = $this->clean($order->get_billing_postcode());
      $data['city'] = $this->clean($order->get_billing_city());
      $data['address_1'] = $this->clean($order->get_billing_address_1());
      $data['country'] = $this->clean($order->get_billing_country());
    }
    if ( empty($data['name']) ) $data['name'] = $this->clean($order->get_billing_first_name());
    if ( empty($data['surname']) ) $data['surname'] = $this->clean($order->get_billing_last_name());
    if ( empty($data['phone']) ) $data['phone'] = $this->clean($order->get_billing_phone());
    
    //Fix postcode
    $data['postcode'] = preg_replace("/[^0-9]/", "", $data['postcode']);
    if ($data['country'] == 'LV') {
      $data['postcode'] = 'LV-' . $data['postcode'];
    }

    return ($object) ? (object) $data : $data;
  }

  private function get_order_weight($id_order)
  {
    $weight_unit = get_option('woocommerce_weight_unit');
    $weight = get_post_meta($id_order, '_cart_weight', true);
    if ( $weight_unit != 'kg' ) {
      $weight = wc_get_weight($weight, 'kg', $weight_unit);
    }

    return $weight;
  }

  public function get_send_method($order)
  {
    $send_method = '';
    foreach ( $order->get_items('shipping') as $item_id => $shipping_item_obj ) {
      $send_method = $shipping_item_obj->get_method_id();
    }
    if ( $send_method == 'manspasts' ) {
      $send_method = get_post_meta($order->get_id(), '_post_method', true);
    }else{
      $send_method = $shipping_item_obj->get_method_id();
    }
    if ($send_method == 'manspasts_m') $send_method = 'm'; //TODO: Make dynamicaly
    if ($send_method == 'manspasts_mp') $send_method = 'mp';

    return $send_method;
  }

  protected static function getReferenceNumber($order_number)
  {
    $order_number = (string) $order_number;
    $kaal = array(7, 3, 1);
    $sl = $st = strlen($order_number);
    $total = 0;
    while ( $sl > 0 and substr($order_number, --$sl, 1) >= '0' ) {
      $total += substr($order_number, ($st - 1) - $sl, 1) * $kaal[($sl % 3)];
    }
    $kontrollnr = ((ceil(($total / 10)) * 10) - $total);
    
    return $order_number . $kontrollnr;
  }

  private function makeReadableXmlResponse($xmlResponse)
  {
    $xmlResponse = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns3:'], '', $xmlResponse);
    $xml = simplexml_load_string($xmlResponse);

    // Another possible preparation variant
    /*$xmlWithNamespaces = simplexml_load_string($xmlResponse);
    $xml = str_replace(array_map(function($e) { return "$e:"; }, array_keys($xmlWithNamespaces->getNamespaces(true))), array(), $xmlResponse);*/

    return $xml;
  }

  private function get_xml_error_from_response($response)
  {
    if ( strpos($response, 'HTTP Status 401') !== false
      && strpos($response, 'This request requires HTTP authentication.') !== false ) {
      return __('Bad API logins', 'manspasts');
    }
    
    return __('Response is in the wrong format', 'manspasts');
  }

  public function clean($string) {
    return str_replace('"',"'",$string);
  }
  private function send($url, $data = null, $checkCert = true) {
     $curl = curl_init();
     curl_setopt($curl, CURLINFO_HEADER_OUT, true);
     curl_setopt($curl, CURLOPT_HEADER, false);
     curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
     curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
     //curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, $checkCert);
    //curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, $checkCert);
    curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
     curl_setopt($curl, CURLOPT_POSTREDIR, true);
     curl_setopt($curl, CURLOPT_URL, $url);
     curl_setopt($curl, CURLOPT_POST, true);
     if (is_string($data)) {
     $data = json_decode($data, true);
     }
    if (empty($data) || !is_array($data)) {
     return false;
     }
     // Content-Type: application/x-www-form-urlencoded
     $data = http_build_query($data);
     curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
     $response = curl_exec($curl);
     $errCode = curl_errno($curl);
     $errText = curl_error($curl);
     curl_close($curl);
     if (!empty($errCode)) {
     $response = array(
     'code' => $errCode,
     'text' => $errText
     );
     }
    return $response;
  }
}
