<?php
class Manspasts_Core {
	public static $main_file_path = WP_PLUGIN_DIR . '/' . MANSPASTS_BASENAME;

	public static function init() {
		self::load_classes();
		self::load_init_hooks();

		if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
			self::load_launch_hooks();
		}
	}

	public static function get_configs($section_name = false) {
		$default_configs = self::default_configs();
		$configs = $default_configs;

		if (function_exists('Manspasts_configs')) {
			foreach (manspasts_configs() as $key => $values) {

				if (is_array($values)) {
					if (!isset($configs[$key])) {
						$configs[$key] = array();
					}
					$configs[$key] = array_merge($configs[$key], $values);
				} else {
					$configs[$key] = $values;
				}
			}
		}

		if ($section_name && isset($configs[$section_name])) {
			$configs = $configs[$section_name];
		}

		return $configs;
	}

	public static function get_settings() {
		return get_option('woocommerce_manspasts_settings');
	}

	public static function textdomain() {
		load_plugin_textdomain('Manspasts', false, dirname(MANSPASTS_BASENAME) . '/languages');
	}

	public static function add_required_directories() {
		$directories = array('logs', 'debug');

		foreach ($directories as $dir) {
			if (!file_exists(MANSPASTS_DIR . 'var/' . $dir)) {
				mkdir(MANSPASTS_DIR . $dir, 0775, true);
			}
		}
	}

	public static function get_overrides_dir($get_url = false) {
		$directory = '/manspasts/';
		if ($get_url) {
			return get_template_directory_uri() . $directory;
		}

		return get_template_directory() . $directory;
	}

	public static function check_update($current_version = '') {
		$update_params = self::get_configs('update');

		if (empty($update_params['check_url'])) {
			return false;
		}

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $update_params['check_url']);
		curl_setopt($ch, CURLOPT_USERAGENT, 'Awesome-Octocat-App');
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$response_data = json_decode(curl_exec($ch));
		curl_close($ch);

		if (isset($response_data->tag_name)) {
			$update_info = array(
				'version' => str_replace('v', '', $response_data->tag_name),
				'url' => (isset($response_data->html_url)) ? $response_data->html_url : '#',
			);
			if (empty($current_version)) {
				$plugin_data = get_file_data(self::$main_file_path, array('Version' => 'Version'), false);
				$current_version = $plugin_data['Version'];
			}
			return (version_compare($current_version, $update_info['version'], '<')) ? $update_info : false;
		}

		return false;
	}

	public static function update_message($file, $plugin) {
		$check_update = self::check_update($plugin['Version']);
		$update_params = self::get_configs('update');

		if ($check_update) {
			echo '<tr class="plugin-update-tr installer-plugin-update-tr js-otgs-plugin-tr active">';
			echo '<td class="plugin-update" colspan="100%">';
			echo '<div class="update-message notice inline notice-warning notice-alt">';
			echo '<p>' . sprintf(__('A newer version of the plugin (%s) has been released.', 'Manspasts'), '<a href="' . $check_update['url'] . '" target="_blank">v' . $check_update['version'] . '</a>');
			if (!empty($update_params['download_url'])) {
				echo ' ' . sprintf(__('You can download it by pressing %s.', 'Manspasts'), '<a href="' . $update_params['download_url'] . '">' . __('here', 'Manspasts') . '</a>');
			}
			if (defined('Manspasts_CUSTOM_CHANGES') && !empty(MANSPASTS_CUSTOM_CHANGES)) {
				echo '<br/><strong style="color:red;">' . __('We do not recommend update the plugin, because your plugin have changes that is not included in the update', 'Manspasts') . ':</strong>';
				foreach (MANSPASTS_CUSTOM_CHANGES as $change) {
					echo '<br/>· ' . $change . '';
				}
			}
			echo '</p>';
			echo '</div>';
			echo '</td></tr>';
		}
	}

	public static function load_front_scripts() {
		$folder_css = '/assets/css/';
		$folder_js = '/assets/js/';

		if (is_cart() || is_checkout()) {
			wp_enqueue_script('Manspasts-helper', plugins_url($folder_js . 'Manspasts_helper.js', self::$main_file_path), array('jquery'), MANSPASTS_VERSION);
			wp_enqueue_style('Manspasts', plugins_url($folder_css . 'Manspasts.css', self::$main_file_path), array(), MANSPASTS_VERSION);

			if (file_exists(MANSPASTS_DIR . $folder_css . 'custom.css')) { //Allow custom CSS file which not include in plugin by default
				wp_enqueue_style('Manspasts-custom', plugins_url($folder_css . 'custom.css', self::$main_file_path), array(), MANSPASTS_VERSION);
			}
			if (file_exists(self::get_overrides_dir() . 'assets/css/front.css')) { //Allow custom CSS file from theme 
				wp_enqueue_style('Manspasts-theme-front', self::get_overrides_dir(true) . 'css/front.css', array(), MANSPASTS_VERSION);
			}


			wp_localize_script('manspasts', 'manspastsdata', array(
				'ajax_url' => admin_url('admin-ajax.php'),
				'Manspasts_plugin_url' => MANSPASTS_URL,
				'text_select_terminal' => __('Select terminal', 'Manspasts'),
				'text_select_post' => __('Select post office', 'Manspasts'),
				'text_search_placeholder' => __('Enter postcode', 'Manspasts'),
				'not_found' => __('Place not found', 'Manspasts'),
				'text_enter_address' => __('Enter postcode/address', 'Manspasts'),
				'text_show_in_map' => __('Show in map', 'Manspasts'),
				'text_show_more' => __('Show more', 'Manspasts'),
				'text_modal_title_terminal' => __('Manspasts parcel terminals', 'Manspasts'),
				'text_modal_search_title_terminal' => __('Parcel terminals addresses', 'Manspasts'),
				'text_modal_title_post' => __('Manspasts offices', 'Manspasts'),
				'text_modal_search_title_post' => __('Manspasts offices addresses', 'Manspasts'),
				'text_modal_title_circle' => __('Manspasts offices', 'Manspasts'),
				'text_modal_title_courier_plus' => __('Manspasts offices', 'Manspasts'),
				'text_modal_search_title_circle' => __('Manspasts offices addresses', 'Manspasts'),
				'text_select_circle' => __('Select post office', 'Manspasts'),
			));
		}
	}

	public static function load_admin_settings_scripts($hook) {
		$folder_css = '/assets/css/';
		$folder_js = '/assets/js/';

		if ($hook == 'woocommerce_page_wc-settings' && isset($_GET['section']) && $_GET['section'] == 'manspasts') {
			wp_enqueue_style('Manspasts_admin_settings', plugins_url($folder_css . 'manspasts_admin_settings.css', self::$main_file_path), array(), MANSPASTS_VERSION);
			wp_enqueue_script('Manspasts_admin_settings', plugins_url($folder_js . 'Manspasts_admin_settings.js', self::$main_file_path), array('jquery'), MANSPASTS_VERSION);
		}
	}

	public static function add_asyncdefer_by_handle($tag, $handle) {
		if (strpos($handle, 'async') !== false) {
			$tag = str_replace('<script ', '<script async ', $tag);
		}
		if (strpos($handle, 'defer') !== false) {
			$tag =  str_replace('<script ', '<script defer ', $tag);
		}

		return $tag;
	}

	public static function init_shipping_method() {
		require_once MANSPASTS_DIR . 'core/class-shipping-method-helper.php';
		include MANSPASTS_DIR . 'core/class-shipping-method.php';
	}

	public static function add_shipping_method($methods) {
		$methods['Manspasts'] = 'Manspasts_Shipping_Method';
		return $methods;
	}

	public static function settings_link($links) {
		array_unshift($links, '<a href="' . admin_url('admin.php?page=wc-settings&tab=shipping&section=manspasts') . '">' . __('Settings', 'Manspasts') . '</a>');
		return $links;
	}

	public static function admin_notices() {
		if (!session_id()) {
			session_start();
		}
		if (array_key_exists('manspasts_notices', $_SESSION)) {
			foreach ($_SESSION['manspasts_notices'] as $notice) {
				echo '<div class="' . $notice['type'] . '"><p>' . $notice['msg'] . '</p></div>';
			}
			unset($_SESSION['manspasts_notices']);
		}
	}

	public static function add_to_footer() {
		if (is_cart() || is_checkout()) {
			echo Manspasts_Terminals::terminals_modal();
		}
	}

	public static function get_error_text($error_code) {
		$errors = array(
			'001' => _x('The service is not available to the sender country', 'error', 'Manspasts'),
			'002' => _x('The service is not available to the receiver country', 'error', 'Manspasts'),
			'003' => _x('This shipping set does not exist', 'error', 'Manspasts'),
			'004' => _x('The service does not exist for the specified method', 'error', 'Manspasts'),
		);

		if (!isset($errors[$error_code])) {
			return _x('Unknown error', 'error', 'Manspasts');
		}

		return $errors[$error_code];
	}

	private static function load_classes() {
		$core_dir = MANSPASTS_DIR . 'core/';
		require_once $core_dir . 'class-debug.php';
		require_once $core_dir . 'class-helper.php';
		require_once $core_dir . 'class-emails.php';
		require_once $core_dir . 'class-labels.php';
		require_once $core_dir . 'class-api.php';
		require_once $core_dir . 'class-admin-html.php';
		require_once $core_dir . 'class-packer.php';
		require_once $core_dir . 'class-terminals.php';
		require_once $core_dir . 'class-manifest.php';
		require_once $core_dir . 'class-order.php';
		require_once $core_dir . 'class-frontend.php';
	}

	private static function default_configs() {
		return array(
			'settings_key' => 'woocommerce_manspasts_settings',
			'method_params' => array(),
			'locations' => array(),
			'update' => array(),
			'text_variables' => array(),
			'debug' => array(
				'delete_after' => 30,
			),
			'meta_keys' => array(
				'method' => '_manspasts_method',
				'barcode' => '_manspasts_barcode',
				'error' => '_manspasts_error',
				'manifest_date' => '_manspasts_manifest_date',
				'terminal_id' => '_manspasts_terminal_id',
			),
		);
	}

	private static function load_init_hooks() {
		add_action('woocommerce_shipping_init', 'Manspasts_Core::init_shipping_method');
		add_action('init', 'Manspasts_Core::textdomain');
		add_action('admin_notices', 'Manspasts_Core::admin_notices');
		add_action('after_plugin_row_' . MANSPASTS_BASENAME, 'Manspasts_Core::update_message', 10, 3);

		add_filter('plugin_action_links_' . MANSPASTS_BASENAME, 'Manspasts_Core::settings_link');
	}

	private static function load_launch_hooks() {
		add_action('wp_enqueue_scripts', 'Manspasts_Core::load_front_scripts', 99);
		add_action('manspasts_admin_manifest_head', 'Manspasts_Manifest::load_admin_scripts');
		add_action('admin_enqueue_scripts', 'Manspasts_Core::load_admin_settings_scripts');
		add_action('admin_enqueue_scripts', 'Manspasts_Order::load_admin_scripts');
		add_action('wp_footer', 'Manspasts_Core::add_to_footer');
		add_action('wp_ajax_nopriv_add_terminal_to_session', 'Manspasts_Terminals::add_terminal_to_session');
		add_action('wp_ajax_add_terminal_to_session', 'Manspasts_Terminals::add_terminal_to_session');
		add_action('admin_menu', 'Manspasts_Manifest::register_menu_pages');
		add_action('woocommerce_after_shipping_rate', 'Manspasts_Order::after_rate_description', 20, 2);
		add_action('woocommerce_checkout_update_order_meta', 'Manspasts_Order::add_terminal_id_to_order');
		add_action('woocommerce_review_order_before_cart_contents', 'Manspasts_Order::validate_order', 10);
		add_action('woocommerce_after_checkout_validation', 'Manspasts_Order::validate_order', 10);
		add_action('woocommerce_order_details_after_order_table', 'Manspasts_Order::show_selected_terminal', 10, 1);
		add_action('woocommerce_email_after_order_table', 'Manspasts_Order::show_selected_terminal', 10, 1);
		add_action('woocommerce_admin_order_preview_end', 'Manspasts_Order::display_order_data_in_admin');
		add_action('wp_ajax_generate_manspasts_label', 'Manspasts_Order::generate_label');
		add_action('woocommerce_checkout_update_order_meta', 'Manspasts_Order::add_cart_weight');
		add_action('woocommerce_admin_order_data_after_shipping_address', 'Manspasts_Order::admin_order_display', 10, 2);
		add_action('save_manspasts', 'Manspasts_Order::admin_order_save');
		add_action('woocommerce_checkout_process', 'Manspasts_Order::checkout_validate_terminal');

		add_filter('script_loader_tag', 'Manspasts_Core::add_asyncdefer_by_handle', 10, 2);
		add_filter('woocommerce_shipping_methods', 'Manspasts_Core::add_shipping_method');
		add_filter('woocommerce_order_data_store_cpt_get_orders_query', 'Manspasts_Manifest::handle_custom_query_var', 10, 2);
		//add_filter('woocommerce_package_rates', 'Manspasts_Order::restrict_shipping_methods_by_cats', 10, 1);
		add_filter('woocommerce_admin_order_preview_get_order_details', 'Manspasts_Order::admin_order_add_custom_meta_data', 10, 2);
		add_filter('bulk_actions-edit-shop_order', 'Manspasts_Order::bulk_actions', 20);
		add_filter('handle_bulk_actions-edit-shop_order', 'Manspasts_Order::handle_bulk_actions', 20, 3);
		add_filter('admin_post_manspasts_labels', 'Manspasts_Order::manspasts_label_actions', 20, 3);
		add_filter('admin_post_manspasts_manifest', 'Manspasts_Order::manspasts_manifest_actions', 20, 3);
		add_filter('woocommerce_cart_shipping_method_full_label', 'Manspasts_Frontend::add_logo_to_method', 10, 2);
	}
}
