<?php
class Manspasts_Labels {
	private $manspasts_settings;
	private $manspasts_configs;
	private $eu_countries;
	private $send_methods = array();

	public function __construct() {
		$this->manspasts_configs = Manspasts_Core::get_configs();
		$this->manspasts_settings = Manspasts_Core::get_settings();
		$this->eu_countries = WC()->countries->get_european_union_countries();

		foreach ($this->manspasts_configs['method_params'] as $method_name => $method_values) {
			if (!$method_values['is_shipping_method']) continue;
			$this->send_methods[$method_values['key']] = 'manspasts_' . $method_values['key'];
		}
	}

	public function create_parcel($orderId, $regenerate = false) {
		$api_url = $this->manspasts_settings['test_mode'] ? 'apidemo.manspasts.lv' : 'www.manspasts.lv';
		$user = $this->manspasts_settings['api_user'];
		$apikey = $this->manspasts_settings['api_pass'];
		$hs_code = $this->manspasts_settings['hs_code'];
		$origin_country_id = $this->manspasts_settings['origin_country_id'];
		$error = array();

		$barcode = get_post_meta($orderId, $this->manspasts_configs['meta_keys']['barcode'], true);
		if (!$barcode || $regenerate) {
			$wc_order = wc_get_order((int)$orderId);
			foreach ($wc_order->get_items('shipping') as $item_id => $item) {
				$method_key_id = $item['method_id'] . '_' . $item['instance_id'];
				$option_name = 'woocommerce_' . $method_key_id . '_settings';
				$option = get_option($option_name, true);
				$ptype = explode('_', $option['item_type-postage_type']);
			}
			$country = $wc_order->get_shipping_country();

			$item_type = $ptype[0] ?? '';
			$postage_type = $ptype[1] ?? '';
			$phone = $wc_order->get_shipping_phone() ?: $wc_order->get_billing_phone();
			$shipment_data = array(
				"package_create" => array(
					"user" => $user,
					"api_key" => $apikey,
					"type" => 'Goods',
					"postageType" => $postage_type,
					"itemType" => $item_type,
					"addresses" => array(
						array(
							"countryCode" => mb_strtoupper($country),
							"postCode" => $wc_order->get_shipping_postcode(),
							"freeformAddressLine1" => $wc_order->get_shipping_address_1(),
							"freeformAddressLine2" => $wc_order->get_shipping_address_2() . ' ' . $wc_order->get_shipping_city(),
							"name" => $wc_order->get_shipping_first_name() . ' ' . $wc_order->get_shipping_last_name(),
							"phone" => $phone ?: '',
							"email" => $wc_order->get_billing_email() ?: '',
						)
					)
				)
			);

			if (in_array($country, $this->eu_countries) && $country != 'LV') {
				$total_weight = 0;
				foreach ($wc_order->get_items() as $prod) {
					$p = $prod->get_product();
					if ($p->get_weight() !== '') {
						$qty = $prod->get_quantity();
						$total_weight += floatval($qty * $p->get_weight());
					}
				}
				if ($total_weight == 0) {
					$total_weight = 0.01;
				}
				$shipment_data['package_create']['addresses'][0]['userPackageWeight'] = $total_weight;
			}

			// Add state to US shipment
			if ($country == 'US') {
				$shipment_data['package_create']['addresses'][0]['freeformAddressLine2'] .= ', ' . $wc_order->get_shipping_state();
			}

			// Add product data to non-EU country orders
			if (!in_array($country, $this->eu_countries)) {
				$content_items = array();
				foreach ($wc_order->get_items() as $prod) {
					$p = $prod->get_product();
					if ($p->get_weight() !== '') {
						$qty = $prod->get_quantity();
						$weight = ($qty * $p->get_weight());
					} else {
						$weight = 0.001;
					}

					$content_items[] = array(
						'name' => $prod->get_name(),
						'quantity' => (int)$prod->get_quantity(),
						'weight' => (int)($weight * 1000),
						'value' => floatval($prod->get_total()),
						'hsCode' => $hs_code,
						'country' => $origin_country_id,
					);
				}

				$shipment_data['package_create']['addresses'][0]['contentItems'] = $content_items;
				$shipment_data['package_create']['addresses'][0]['contentType'] = 'Other';
			}

			$url = $api_url . '/api/packages';
			$response1 = $this->send($url, $shipment_data);
			$response = json_decode($response1, true);

			//Debug
			$shipment_data['url'] = $url;
			Manspasts_Debug::debug_write($shipment_data, "CREATE PARCEL REQUEST");
			$log_resp = (json_last_error() != 0) ? substr(strip_tags($response1), 0, 100) : $response;
			Manspasts_Debug::debug_write($log_resp, "CREATE PARCEL RESPONSE");

			if (!empty($response['code'])) {
				if ($response['code'] == 201 && !empty($response['barcodes'][0])) {
					update_post_meta($orderId, '_manspasts_barcode', $response['barcodes'][0]);
					update_post_meta($orderId, '_manspasts_manifest_link', '');
					update_post_meta($orderId, '_manspasts_manifest_date', '');
					update_post_meta($orderId, $this->manspasts_configs['meta_keys']['error'], '');
				} else {
					$error[$orderId] = $response['message'] ?? $response1;
					$error[$orderId] .= !empty($response['data']['required']) ? ': '.$response['data']['required'] : '';
					update_post_meta($orderId, $this->manspasts_configs['meta_keys']['error'], $error[$orderId]);
				}
			} else {
				if(json_last_error() != 0) {
					$error[$orderId] = substr(strip_tags($response1), 0, 100);
				}else{
					$error[$orderId] = $response['message'] ?? $response1;
					$error[$orderId] .= !empty($response['data']['required']) ? ': '.$response['data']['required'] : '';
				}
				update_post_meta($orderId, $this->manspasts_configs['meta_keys']['error'], $error[$orderId]);
			}
		}

		return $error;
	}

	public function print_labels($orderId, $regenerate = false)	{
		$error = array();
		$error = $this->create_parcel($orderId, $regenerate);

		$api_url = $this->manspasts_settings['test_mode'] ? 'apidemo.manspasts.lv' : 'www.manspasts.lv';
		$user = $this->manspasts_settings['api_user'];
		$apikey = $this->manspasts_settings['api_pass'];

		if (empty($error)) {
			$url = $api_url . '/api/documents';
			$wc_order = wc_get_order((int)$orderId);
			$country = $wc_order->get_shipping_country();
			$barcode = get_post_meta($orderId, $this->manspasts_configs['meta_keys']['barcode'], true);

			$label_data = array(
				"document" => array(
					"user" => $user,
					"api_key" => $apikey,
					"barcode" => $barcode,
					"documentType" => ($country == "LV") ? "Address labels" : "Accompanying documents",
					"addressLabelType" => "Addressee",
					"addressLabelLayout" => "3x8",
					"addressLabelDimensions" => "70x35",
					"addressLabelInitialColumn" => 1,
					"addressLabelInitialRow" => 1
				)
			);

			// Print label in A4 if outside EU and Item type == Parcel
			if (!in_array($country, $this->eu_countries) && mb_strtolower($item_type) == 'parcel') {
				$label_data['document']['documentPrintType'] = 'A4';
			}

			$response1 = $this->send($url, $label_data);
			$response = json_decode($response1, true);

			//Debug
			$label_data['url'] = $url;
			Manspasts_Debug::debug_write($label_data, "LABELS REQUEST");
			$log_resp = (json_last_error() != 0) ? substr(strip_tags($response1), 0, 100) : $response;
			Manspasts_Debug::debug_write($log_resp, "LABELS RESPONSE");

			if (!empty($response['link'])) {
				$pdfname = 'manspasts_label' . time() . '.pdf';
				$file_url = $response['link'];
				header('Content-Type: application/pdf');
				header("Content-Transfer-Encoding: Binary");
				header("Content-disposition: attachment; filename=" . $pdfname);
				readfile($file_url);
			} else {
				$err_msg = sprintf(__('Can\'t generate %s. API response: ', 'manspasts'), 'labels');
				$err_api = $response['message'] ?? json_encode($response1, JSON_UNESCAPED_UNICODE);
				Manspasts_Helper::add_msg($err_msg . $err_api, 'error');
				wp_safe_redirect(wp_get_referer());
			}
		} else {
			foreach ($error as $id => $err) {
				Manspasts_Helper::add_msg('Order #' . $id . ' - ' . $err, 'error');
			}
			wp_safe_redirect(wp_get_referer());
		}
	}

	public function clean($string) {
		return str_replace('"', "'", $string);
	}

	public function send($url, $data)	{
		$curl = curl_init();
		curl_setopt($curl, CURLINFO_HEADER_OUT, true);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
		//curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, $checkCert);
		//curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, $checkCert);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($curl, CURLOPT_POSTREDIR, true);
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_POST, true);

		// Attach encoded JSON string to the POST fields
		curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));

		// Set the content type to application/json
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));

		// Return response instead of outputting
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		$response = curl_exec($curl);
		$errCode = curl_errno($curl);
		$errText = curl_error($curl);
		curl_close($curl);
		if (!empty($errCode)) {
			$response = array(
				'code' => $errCode,
				'text' => $errText
			);
		}
		return $response;
	}
}
