<?php
class Manspasts_Order {
	public static function load_admin_scripts($hook) {
		global $manspasts;

		if ($hook == 'manspasts-new.php' || $hook == 'manspasts.php') {
			if ($manspasts->post_type === 'shop_order') {
				wp_enqueue_script('post_admin_order', plugins_url('/assets/js/post_admin_order.js', Manspasts_Core::$main_file_path), array('jquery'), POST_VERSION);
			}
		}
	}

	public static function after_rate_description($method, $index) {
		if (is_cart()) return; // Exit on cart page

		$customer = WC()->session->get('customer');
		if (!isset($customer['country'])) {
			return;
		}

		$shipping_settings = Manspasts_Core::get_settings();
		if (!isset($shipping_settings['prices_' . $customer['country']])) {
			return;
		}

		$rate_settings = json_decode($shipping_settings['prices_' . $customer['country']]);
		$shipping_methods = Manspasts_Core::get_configs('method_params');

		foreach ($shipping_methods as $ship_method => $ship_method_values) {
			if (!$ship_method_values['is_shipping_method']) continue;

			if (!empty($rate_settings->{$ship_method_values['key'] . "_description"}) && $method->id === 'manspasts_' . $ship_method_values['key']) {
				echo '<span class="post-shipping-description">' . $rate_settings->{$ship_method_values['key'] . "_description"} . '</span>';
			}
		}
	}

	/**
	 * Restrict Post Shipping methods if cart products has restricted categories
	 */
	public static function restrict_shipping_methods_by_cats($rates) {
		global $woocommerce;
		$configs = Manspasts_Core::get_configs();
		$settings = get_option($configs['settings_key']);
		$cart_categories_ids = array();

		foreach ($woocommerce->cart->get_cart() as $cart_item) {
			$cats = get_the_terms($cart_item['product_id'], 'product_cat');
			if (empty($cats)) {
				continue;
			}
			foreach ($cats as $cat) {
				$cart_categories_ids[] = $cat->term_id;
				if ($cat->parent != 0) {
					$cart_categories_ids[] = $cat->parent;
				}
			}
		}
		$cart_categories_ids = array_unique($cart_categories_ids);

		$restricted_categories = $settings['restricted_categories'] ?? array();
		if (!is_array($restricted_categories)) {
			$restricted_categories = array($restricted_categories);
		}

		if(!empty($restricted_categories)) {
			foreach ($cart_categories_ids as $cart_product_categories_id) {
				if (in_array($cart_product_categories_id, $restricted_categories)) {
					foreach ($configs['method_params'] as $ship_method => $ship_method_values) {
						if (!$ship_method_values['is_shipping_method']) continue;
						unset($rates['manspasts_' . $ship_method_values['key']]);
					}
					break;
				}
			}
		}

		return $rates;
	}

	public static function add_terminal_id_to_order($order_id) {
		$configs = Manspasts_Core::get_configs();

		if (isset($_POST['manspasts_terminal']) && $order_id) {
			update_post_meta($order_id, $configs['meta_keys']['terminal_id'], $_POST['manspasts_terminal']);
		}

		if (isset($_POST['shipping_method']) && is_array($_POST['shipping_method'])) {
			foreach ($_POST['shipping_method'] as $ship_method) {
				foreach ($configs['method_params'] as $method_name => $method_values) {
					if (!$method_values['is_shipping_method']) continue;
					if ($ship_method == "manspasts_" . $method_values['key']) {
						update_post_meta($order_id, $configs['meta_keys']['method'], $ship_method);
						break;
					}
				}
			}
		}
	}

	public static function validate_order($manspastsed) {
		$packages = WC()->shipping->get_packages();
		$chosen_methods = WC()->session->get('chosen_shipping_methods');
		if (is_array($chosen_methods) && in_array('manspasts', $chosen_methods)) {
			foreach ($packages as $i => $package) {
				if ($chosen_methods[$i] != 'manspasts') {
					continue;
				}

				$shipping_settings = Manspasts_Core::get_settings();
				$weightLimit = (int) $shipping_settings['weight'];
				$weight = 0;
				foreach ($package['contents'] as $item_id => $values) {
					$_product = $values['data'];
					$weight = $weight + $_product->get_weight() * $values['quantity'];
				}

				$weight = wc_get_weight($weight, 'kg');
				if ($weight > $weightLimit) {
					$message = sprintf(__('Sorry, %d kg exceeds the maximum weight of %d kg for %s', 'manspasts'), $weight, $weightLimit, __('Post shipping', 'manspasts'));
					$messageType = "error";
					if (!wc_has_notice($message, $messageType)) {
						wc_add_notice($message, $messageType);
					}
				}
			}
		}
	}

	public static function show_selected_terminal($order) {
		$configs_methods = Manspasts_Core::get_configs('method_params');
		$send_method = self::get_post_method($order);

		foreach ($configs_methods as $method_key => $method_values) {
			if (!$method_values['is_shipping_method']) continue;
			if ($send_method != 'manspasts_' . $method_values['key']) continue;

			if ($method_values['key'] == 'm' || $method_values['key'] == 'mp') {
				echo '<p>Post ' . strtolower($method_values['title']) . ": " . Manspasts_Terminals::get_terminal_address($order) . '</p>';
			}
		}
	}

	public static function show_tracking_link($order) {
		$send_method = self::get_post_method($order);

		if ($send_method) {
			//$barcode = $order->get_meta($this->manspasts_configs['meta_keys']['barcode']);
			$barcode = $order->get_meta('_manspasts_barcode');
			$html = '';
			if (!empty($barcode)) {
				$text = __('Manspasts tracking number', 'manspasts');
				$html = '<p><strong>' . $text . ':</strong> <br/>' . $barcode . '</p>';
			}
			echo $html;
		}
	}

	/**
	 * Add custom order meta data to make it accessible in Order preview template
	 */
	public static function admin_order_add_custom_meta_data($data, $order) {
		$configs = Manspasts_Core::get_configs();
		$send_method = self::get_post_method($order);

		foreach ($configs['method_params'] as $method_key => $method_values) {
			if (!$method_values['is_shipping_method']) continue;
			if ($send_method != 'manspasts_' . $method_values['key']) continue;

			if ($method_values['key'] == 'mp' || $method_values['key'] == 'm') {
				$data['shipping_via'] = 'manspasts ' . strtolower($method_values['title']) . ": " . Manspasts_Terminals::get_terminal_address($order);
			}
		}

		if ($send_method) {
			$barcode = $order->get_meta($configs['meta_keys']['barcode']);
			$data['manspasts_barcode'] = $barcode;
		}

		return $data;
	}

	public static function display_order_data_in_admin() {
		echo '<# if ( data.manspasts_barcode ) { #>' .
			'<p><div class="wc-order-preview-addresses">' .
			'<div class="wc-order-preview-address">' .
			'<strong>' . __('Post tracking number', 'manspasts') . ':</strong>{{data.manspasts_barcode}}' .
			'</div></div></p>' .
			'<# } #>';
	}

	public static function bulk_actions($actions) {
		$actions['manspasts_Labels'] = __('Print Post labels', 'manspasts');
		$actions['manspasts_manifest'] = __('Print Post manifest', 'manspasts');
		return $actions;
	}

	public static function handle_bulk_actions($redirect_to, $action, $ids) {
		if ($action == "manspasts_labels") {
			$Manspasts_Labels = new Manspasts_Labels();
			$Manspasts_Labels->print_labels($ids);
			return 0;
		}
		if ($action == "manspasts_manifest") {
			$wc_shipping = new WC_Shipping();
			$manspasts = new Manspasts_Shipping_Method();
			$manspasts->printBulkManifests();
			return 0;
		}

		return $redirect_to;
	}

	public static function manspasts_label_actions() {
		$regenerate = false;
		if (isset($_REQUEST['process']) && $_REQUEST['process'] === 'regenerate') {
			$regenerate = true;
		}

		$Manspasts_Labels = new Manspasts_Labels();
		$Manspasts_Labels->print_labels($_REQUEST['post'], $regenerate);
	}

	public static function manspasts_manifest_actions() {
		$wc_shipping = new WC_Shipping();
		$manspasts = new Manspasts_Shipping_Method();
		$manspasts->printBulkManifests();
	}

	/*public static function order_actions($order) //Disabled because not found where to using (possible junk from old plugin versions)
  {
    $order_id = $order->get_id();
    $send_method = get_post_meta($order_id, '_shipping_method', true);
    $configs_methods = Manspasts_Core::get_configs('method_params');

    foreach ( $configs_methods as $method_name => $method_values ) {
      if ( ! $method_values['is_shipping_method'] ) continue;

      if ( isset($send_method[0]) && $send_method[0] == 'manspasts_' . $method_values['key'] ) {
        echo '<a class="button tips post_generate_label" href="' . wp_nonce_url(admin_url('admin-ajax.php?action=generate_post_label&order_id=' . $order_id), 'woocommerce-mark-order-status') . '" data-tip="' . __('Generate Post label', 'manspasts') . '"> </a>';
        if ( file_exists(POST_DIR . "var/pdf/" . $order_id . '.pdf') ) {
          echo '<a class="button tips post_view_label" href="' . plugins_url('var/pdf/' . $order_id . '.pdf', Manspasts_Core::$main_file_path) . '" target = "_blank" data-tip="' . __('View Post label', 'manspasts') . '"> </a>';
        }
      }
    }
  }*/

	public static function generate_label() {
		if (current_user_can('edit_shop_orders') && check_admin_referer('woocommerce-mark-order-status')) {
			$Manspasts_Labels = new Manspasts_Labels();
			$Manspasts_Labels->print_labels($_GET['order_id'], false);
		}
		wp_safe_redirect(wp_get_referer() ? wp_get_referer() : admin_url('edit.php?post_type=shop_order'));
		exit;
	}

	/**
	 * Add weight to order
	 */
	public static function add_cart_weight($order_id) {
		global $woocommerce;
		$weight = $woocommerce->cart->cart_contents_weight;
		update_post_meta($order_id, '_cart_weight', $weight);
	}

	/**
	 * Display and edit Post info on the order edit page
	 */
	public static function admin_order_display($order, $print_barcode = true, $admin_panel = true) {
		$configs = Manspasts_Core::get_configs();
		$wc_order = wc_get_order((int) $order->get_id());
		$send_method = "";
		foreach ($wc_order->get_items('shipping') as $item_id => $shipping_item_obj) {
			$send_method = $shipping_item_obj->get_method_id();
		}
		$Manspasts_Labels = new Manspasts_Labels();

		$is_post = false;
		foreach ($configs['method_params'] as $ship_method => $ship_values) {
			if (!$ship_values['is_shipping_method']) continue;

			if ($send_method == 'manspasts_' . $ship_values['key']) {
				$is_post = true;
			}
		}

		global $manspasts_type;
		$only_in_order = true;
		if ('shop_order' != $manspasts_type) {
			$only_in_order = false;
		}

		if ($only_in_order) {
			echo '<br class="clear"/>';
			echo '<hr style="margin-top:20px;">';
			echo '<h4>' . __('Post shipping', 'manspasts') . '</h4>';
		}

		echo '<div class="address">';
		foreach ($configs['method_params'] as $ship_method => $ship_values) {

			if ($send_method == 'manspasts' || $send_method == 'manspasts_mm') {
				$field_value = $order->get_formatted_shipping_address();
				echo '<p><strong class="title">' . __('MansPasts', 'manspasts') . ':</strong> ' . $field_value . '</p>';
			}
		}

		if ($print_barcode) {
			$barcode = $order->get_meta('_manspasts_barcode') ?? '';
			echo $barcode;
		}
		echo '</div>';

		if (!$only_in_order) {
			return;
		}

		echo '<div class="edit_address">';
		if ($send_method == 'manspasts_manspasts' || $send_method == 'manspasts_mp') {
			$values = array(
				'terminal_key' => 'manspasts_plus',
				'change_title' => __('Change parcel terminal', 'manspasts'),
			);
			if ($send_method == 'manspasts_m') {
				$values['terminal_key'] = 'manspasts';
				$values['change_title'] = __('Change post office', 'manspasts');
			}

			$all_terminals = Manspasts_Terminals::get_terminals_list('ALL', $values['terminal_key']);
			$selected_terminal = get_post_meta($order->get_id(), $configs['meta_keys']['terminal_id'], true);

			echo '<p class="form-field-wide">';
			echo '<label for="post_terminal">' . $values['change_title'] . '</label>';
			echo '<input type="hidden" id="post-order-country" value="' . $order->get_shipping_country() . '">';
			echo '<select id="post_terminal" class="select short" name="post_terminal_id">';
			echo '<option>-</option>';
			foreach ($all_terminals as $country => $country_terminals) {
				foreach ($country_terminals as $county => $terminals) {
					echo '<optgroup data-country="' . $country . '" label="' . $county . '">';
					foreach ($terminals as $terminal_id => $terminal_name) {
						$selected = ($terminal_id == $selected_terminal) ? 'selected' : '';
						echo '<option value="' . $terminal_id . '" ' . $selected . '>' . $terminal_name . '</option>';
					}
					echo '</optgroup>';
				}
			}
			echo '</select>';
			echo '</p>';
		} else {
			echo __('The delivery address is changed in the fields above', 'manspasts');
		}

		echo '</div>';
		echo '<hr style="margin-top:20px;">';
	}

	public static function admin_order_save($manspasts_id) {
		global $manspasts_type;
		if ('shop_order' != $manspasts_type) {
			return $manspasts_id;
		}

		$configs = Manspasts_Core::get_configs();

		if (isset($_POST['manspasts_terminal_id'])) {
			update_post_meta($manspasts_id, $configs['meta_keys']['terminal_id'], wc_clean($_POST['manspasts_terminal_id']));
		}

		return $manspasts_id;
	}

	public static function checkout_validate_terminal() {
	}

	private static function get_post_method($order) {
		$configs_meta = Manspasts_Core::get_configs('meta_keys');
		$wc_order = wc_get_order((int) $order->get_id());
		$send_method = "";
		foreach ($wc_order->get_items('shipping') as $item_id => $shipping_item_obj) {
			$send_method = $shipping_item_obj->get_method_id();
		}
		if ($send_method == 'manspasts') {
			return get_post_meta($order->get_id(), $configs_meta['method'], true);
		} else {
			return $send_method;
		}
	}
}
