<?php
if (!class_exists('Manspasts_Shipping_Method')) {
	class Manspasts_Shipping_Method extends WC_Shipping_Method {
		/**
		 * Constructor for your shipping class
		 *
		 * @access public
		 * @return void
		 */
		public $errors = array();

		private $manspasts_configs;
		private $methods_asociations;
		private $manspasts_settings;
		public function __construct() {
			$this->id = 'manspasts';
			$this->method_title = __('MansPasts shipping', 'manspasts');
			$this->method_description = __('Shipping methods for MansPasts', 'manspasts');

			$this->manspasts_api = new Manspasts_Api();
			$this->manspasts_configs = Manspasts_Core::get_configs();
			$this->manspasts_settings = Manspasts_Core::get_settings();
			$this->methods_asociations = Manspasts_Helper::get_methods_asociations();

			// Availability, Countries and other required Woocommerce functions
			$this->availability = 'including';
			$this->init();

			$this->title = isset($this->settings['title']) ? $this->settings['title'] : __('MansPasts shipping', 'manspasts');

			// Default values
			foreach ($this->methods_asociations as $key => $name) {
				if (empty($this->settings['method_' . $key])) {
					$this->settings['method_' . $key] = 'no';
				}
			}
		}

		/**
		 * Init your settings
		 *
		 * @access public
		 * @return void
		 */
		function init() {

			// Load the settings API

			$this->init_form_fields();
			$this->init_settings();

			//$this->title = $this->get_option('title');

			// Save settings in admin if you have any defined
			add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
			//$this->update_locations_file();
		}

		public function update_locations_file() {
			$locations = $this->manspasts_configs['locations'];
			$fp = fopen(MANSPASTS_DIR . '/' . "locations.json", "w");
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, $locations['source_url']);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_HEADER, false);
			curl_setopt($curl, CURLOPT_FILE, $fp);
			curl_setopt($curl, CURLOPT_TIMEOUT, 60);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			$data = curl_exec($curl);
			curl_close($curl);
			fclose($fp);
		}

		/**
		 * Load settings form
		 */
		function admin_options() {
?>
			<h2><?php echo $this->method_title; ?></h2>
			<p><?php echo $this->method_description; ?></p>
			<table class="form-table post-settings">
				<?php $this->generate_settings_html(); ?>
			</table>
		<?php
		}
		/**
		 * Define settings field for this shipping
		 * @return void
		 */
		function init_form_fields() {

			$countries_post = @file_get_contents('https://ftp.webdev.lv/pasts/countries.json');
			$post_countries = json_decode($countries_post, true);
			$options_countries = array();
			if (!empty($post_countries)) {
				//Language code based on admin language
				$current_language = get_bloginfo('language');
				switch (mb_strtolower($current_language)) {
					case 'en-gb':
					case 'en':
					case 'en-en':
						$lang = 'en';
						break;
					case 'lv-lv':
					case 'lv':
						$lang = 'lv';
						break;
					case 'ru-ru':
					case 'ru':
						$lang = 'ru';
						break;
					default:
						$lang = 'en';
				}

				foreach ($post_countries as $result) {
					if (isset($result['Country'])) {
						$options_countries[$result['Country']['iso2']] = $result['Country']['title_' . $lang] . ' (' . $result['Country']['iso2'] . ')';
					}
				}
			}

			$fields = array(
				'test_mode' => array(
					'title' => __('Enable Test mode', 'manspasts'),
					'type' => 'checkbox',
					'default' => 'no',
				),
				'hr_api' => array(
					'type' => 'hr',
					'title' => __('API', 'manspasts'),
				),
				'api_user' => array(
					'title' => __('API user', 'manspasts'),
					'type' => 'text',
					'description' => __('Please contact Post for API access codes.', 'manspasts'),
				),
				'api_pass' => array(
					'title' => __('API password', 'manspasts'),
					'type' => 'password',
				),
				'hs_code' => array(
					'title' => __('HS code', 'post'),
					'type' => 'text'
				),
				'origin_country_id' => array(
					'title' => __('Country of Origin', 'post'),
					'type' => 'select',
					'options' => $options_countries,
					'class' => 'checkout-style pickup-point'
				),
			);

			/*
			$fields['hr_settings'] = array(
				'type' => 'hr',
				'title' => __('Shipping methods settings', 'manspasts'),
			);
			foreach ($this->manspasts_configs['method_params'] as $ship_method => $ship_method_values) {
				$field_key = 'weight_' . $ship_method_values['key'];
				if ($ship_method_values['key'] === 'm') {
					$field_key = 'weight';
				}

				$fields[$field_key] = array(
					'title' => sprintf(__('Max cart weight (%1$s) for %2$s', 'manspasts'), 'kg', strtolower($ship_method_values['title'])),
					'type' => 'number',
					'custom_attributes' => array(
						'step' => 0.001,
						'min' => 0
					),
					'description' => sprintf(__('Maximum allowed all cart products weight for %s.', 'manspasts'), strtolower($ship_method_values['title'])),
					'default' => $ship_method_values['weight']['default'],
					'class' => 'manspasts_' . $ship_method,
				);
			}
			$fields['size_pt'] = array(
				'title' => sprintf(__('Max cart size (%s) for terminal', 'manspasts'), get_option('woocommerce_dimension_unit')),
				'type' => 'dimensions',
				'description' => __('Maximum cart size for parcel terminals. Leave all empty to disable.', 'manspasts') . '<br/>' . __('Preliminary cart size is calculated by trying to fit all products by taking their dimensions (boxes) indicated in their settings.', 'manspasts'),
				'class' => 'manspasts_terminal'
			);
			$fields['size_c'] = array(
				'title' => sprintf(__('Max size (%s) for courier', 'manspasts'),get_option('woocommerce_dimension_unit')),
				'type' => 'dimensions',
				'description' => __('Maximum product size for courier. Leave all empty to disable.', 'manspasts') . '<br/>' . __('If the length, width or height of at least one product exceeds the specified values, then it will not be possible to select the courier delivery method for the whole cart.', 'manspasts')
			);
			$fields['restricted_categories'] = array(
				'title' => __('Disable for specific categories', 'manspasts'),
				'type' => 'multiselect',
				'class' => 'wc-enhanced-select',
				'description' => __('Select categories for which you want to disable the Post method.', 'manspasts'),
				'options' => $this->manspasts_get_categories(),
				//'desc_tip' => true,
				'required' => false,
				'custom_attributes' => array(
					'data-placeholder' => __('Select Categories', 'manspasts'),
					'data-name' => 'restricted_categories'
				),
			);
			*/
			$fields['hr_manifest'] = array(
				'type' => 'hr',
				'title' => __('Manifest', 'manspasts'),
			);
			$fields['manifest_enable'] = array(
				'title' => __('Enable manifest print', 'manspasts'),
				'type' => 'checkbox',
				'description' => __('Allow print manifest. Disable this option will hide table manifest column and manifest generation buttons.', 'manspasts'),
				'default' => 'yes',
			);

			$fields['hr_debug'] = array(
				'type' => 'hr',
				'title' => __('Debug', 'manspasts'),
			);
			$fields['debug_mode'] = array(
				'title' => __('Enable debug mode', 'manspasts'),
				'type' => 'checkbox',
				'description' => __('Enable request and response logging.', 'manspasts') . ' ' . sprintf(__('Log files are stored for %d days.', 'manspasts'), $this->manspasts_configs['debug']['delete_after']),
				'default' => ''
			);
			$fields['debugview_request'] = array(
				'type' => 'debug_window',
				'files' => Manspasts_Debug::get_all_files(),
				'title' => __('Logged data', 'manspasts'),
				'class' => 'manspasts_debug'
			);
			$fields['hr_end'] = array(
				'type' => 'hr',
			);
			$this->form_fields = $fields;
		}

		public function generate_hr_html($key, $value) {
			$class = (isset($value['class'])) ? $value['class'] : '';
			$title = '';
			if (!empty($value['title'])) {
				if (!empty($class)) {
					$class .= ' ';
				}
				$class .= 'have_title';
				$title = '<span>' . $value['title'] . '</span>';
			}

			$html = '<tr valign="top"><td colspan="2" class="section_title"><hr class="' . $class . '">' . $title . '</td></tr>';

			return $html;
		}

		public function generate_empty_html($key, $value) {
			$class = (isset($value['class'])) ? $value['class'] : '';

			$html = '<tr valign="top"><td colspan="2" class="' . $class . '"></td></tr>';

			return $html;
		}

		public function generate_string_html($key, $value) {
			$class = (isset($value['class'])) ? $value['class'] : '';

			$html = '<tr valign="top">';
			$html .= '<th scope="row" class="titledesc"><label>' . $value['title'] . '</label></th>';
			$html .= '<td class="forminp"><fieldset><p class="description">' . $value['text'] . '</p></fieldset></td>';
			$html .= '</tr>';

			return $html;
		}

		public function validate_prices_box_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		public function generate_dimensions_html($key, $value) {
			$field_key = $this->get_field_key($key);
			$field_class = (isset($value['class'])) ? $value['class'] : '';

			if ($this->get_option($key) !== '') {
				$dim_values = $this->get_option($key);
				if (is_string($dim_values)) {
					$dim_values = json_decode($this->get_option($key), true);
				}
			} else {
				$dim_values = array();
			}

			ob_start();
		?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<label><?php echo esc_html($value['title']); ?></label>
				</th>
				<td class="forminp">
					<fieldset class="field-dimensions <?php echo $field_class; ?>">
						<input type="number" value="<?php echo $dim_values[0] ?? ''; ?>" id="<?php echo esc_html($field_key); ?>_0" name="<?php echo esc_html($field_key); ?>[0]" min="0.001" step="0.001" placeholder="<?php echo __('Length', 'manspasts'); ?>">
						<span>x</span>
						<input type="number" value="<?php echo $dim_values[1] ?? ''; ?>" id="<?php echo esc_html($field_key); ?>_1" name="<?php echo esc_html($field_key); ?>[1]" min="0.001" step="0.001" placeholder="<?php echo __('Width', 'manspasts'); ?>">
						<span>x</span>
						<input type="number" value="<?php echo $dim_values[2] ?? ''; ?>" id="<?php echo esc_html($field_key); ?>_2" name="<?php echo esc_html($field_key); ?>[2]" min="0.001" step="0.001" placeholder="<?php echo __('Height', 'manspasts'); ?>">
						<span><?php echo get_option('woocommerce_dimension_unit'); ?></span>
						<?php if (!empty($value['description'])) : ?>
							<p class="description"><?php echo __($value['description']); ?></p>
						<?php endif; ?>
					</fieldset>
				</td>
			</tr>
		<?php
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}
		public function validate_dimensions_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		public function validate_label_name_field($key, $value) {
			$values = wp_json_encode($value);
			return $values;
		}

		/**
		 * Get categories for "restricted_categories" field
		 */
		public function Manspasts_get_categories() {
			$cats = $this->get_categories_hierarchy();
			$result = [];

			foreach ($cats as $item) {
				$this->create_categories_list('', $item, $result);
			}

			return $result;
		}

		/**
		 * Makes a list of categories to select from in settings page. array(lowest cat id => full cat path name)
		 */
		private function create_categories_list($prefix, $data, &$results) {
			if ($prefix) {
				$prefix = $prefix . ' &gt; ';
				$results[$data->term_id] = $prefix . $data->name;
			}
			if (!$data->children) {
				$results[$data->term_id] = $prefix . $data->name;

				return true;
			}

			foreach ($data->children as $child) {
				$this->create_categories_list($prefix . $data->name, $child, $results);
			}
		}

		private function get_categories_hierarchy($parent = 0) {
			$taxonomy = 'product_cat';
			$orderby = 'name';
			$hide_empty = 0;

			$args = array(
				'taxonomy'   => $taxonomy,
				'parent'     => $parent,
				'orderby'    => $orderby,
				'hide_empty' => $hide_empty,
			);

			$cats = get_categories($args);
			$children = array();

			foreach ($cats as $cat) {
				$cat->children = $this->get_categories_hierarchy($cat->term_id);
				$children[$cat->term_id] = $cat;
			}

			return $children;
		}

		public function generate_debug_window_html($key, $value) {
			$field_class = (isset($value['class'])) ? $value['class'] : '';
			$files = (isset($value['files'])) ? $value['files'] : array();
			$files_dir = Manspasts_Debug::$_debug_dir;

			ob_start();
			//Ends PHP for easier HTML output
		?>

			<tr class="post-debugview" valign="top">
				<th scope="row" class="titledesc"></th>
				<td class="forminp">
					<fieldset class="field-debug <?php echo $field_class; ?>">
						<span class="title"><?php echo esc_html($value['title']); ?></span>
						<?php if (empty($files)) : ?>
							<textarea readonly rows="2" style="width:100%">- <?php echo __('Debug files still not created', 'manspasts'); ?> -</textarea>
						<?php else : ?>
							<?php foreach ($files as $file_data) : ?>
								<div class="debug-row">
									<?php
									$file_path = $files_dir . $file_data['name'];
									$file = fopen($file_path, 'r');
									if (filesize($file_path) > 0) {
										$file_content = fread($file, filesize($file_path));
									} else {
										$file_content = '- ' . __('File is empty', 'manspasts') . ' -';
									}
									fclose($file);
									if (!empty($file_data['day'])) {
										$date = date("Y-m-d", strtotime($file_data['day']));
									} else {
										$date = __('Date unknown', 'manspasts');
									}
									?>
									<span class="date"><?= esc_html($date); ?></span>
									<textarea readonly rows="11" style="width:100%;display:none;"><?php echo $file_content; ?></textarea>
								</div>
							<?php endforeach; ?>
						<?php endif; ?>
					</fieldset>
				</td>
			</tr>

<?php
			$html = ob_get_contents();
			ob_end_clean();
			return $html;
		}

		public function printBulkManifests() {
			$api_url = $this->manspasts_settings['test_mode'] ? 'apidemo.manspasts.lv' : 'www.manspasts.lv';
			$user = $this->manspasts_settings['api_user'];
			$pass = $this->manspasts_settings['api_pass'];

			$errors = $barcodes = $manifest_url = array();
			$labels = new Manspasts_Labels;
			$orderIds = $_GET['post'];
			if (!is_array($orderIds)) {
				$orderIds = array((int)$orderIds);
			}
			foreach ($orderIds as $orderId) {
				$err = $labels->create_parcel($orderId, false); //Create parcels, if they don't exist
				$errors = array_merge($errors, $err);
			}

			if (empty($errors)) {
				$time = current_time('Y-m-d H:i:s');
				$barcodes = [];
				foreach ($orderIds as $orderId) {
					$barcode = get_post_meta($orderId, $this->manspasts_configs['meta_keys']['barcode'], true);
					$barcodes = array_merge($barcodes, explode(',', $barcode));
					$manifest_url[] = get_post_meta($orderId, '_manspasts_manifest_link', true);
				}

				// Check if all manifest URLs are the same. No need to generate manifest in that case
				$equal_manifests = (count(array_unique($manifest_url)) === 1);
				if ($equal_manifests && !empty($manifest_url[0])) {
					$response['link'] = $manifest_url[0];
				} else {
					$url = $api_url . '/api/combine-ps27';
					$data['combine_ps27'] = array(
						"user" => $user,
						"api_key" => $pass,
						"barcodes" => $barcodes,
					);
					$response = $labels->send($url, $data);

					//Debug
					$data['url'] = $url;
					Manspasts_Debug::debug_write($data, "MANIFEST REQUEST");
					Manspasts_Debug::debug_write($response, "MANIFEST RESPONSE");

					if (!is_array($response)) {
						$response = json_decode($response, true);
						if (!empty($response['link'])) {
							foreach ($orderIds as $orderId) {
								update_post_meta($orderId, '_manspasts_manifest_link', $response['link']);
								update_post_meta($orderId, '_manspasts_manifest_date', $time);
							}
						}
					} else {
						$err_msg = sprintf(__('Can\'t generate %s. API response: ', 'manspasts'), 'manifest');
						$err_api = $response['message'] ?? json_encode($response, JSON_UNESCAPED_UNICODE);
						Manspasts_Helper::add_msg($err_msg . $err_api, 'error');
						wp_safe_redirect(wp_get_referer());
					}
				}

				if (!empty($response['link'])) {
					$pdfname = 'manspasts_manifest' . time() . '.pdf';
					$file_url = $response['link'];
					header('Content-Type: application/pdf');
					header("Content-Transfer-Encoding: Binary");
					header("Content-disposition: attachment; filename=" . $pdfname);
					readfile($file_url);
				} else {
					if (!empty($response['message'])) {
						if (is_array($response['message']) || is_object($response['message'])) {
							$err_api = json_encode($response['message'], JSON_UNESCAPED_UNICODE);
						} else {
							$err_api = $response['message'];
						}
					} else {
						$err_api = json_encode($response, JSON_UNESCAPED_UNICODE);
					}

					$err_msg = sprintf(__('Can\'t generate %s. API response: ', 'manspasts'), 'manifest');
					Manspasts_Helper::add_msg($err_msg . $err_api, 'error');
					wp_safe_redirect(wp_get_referer());
				}
			} else {
				foreach ($errors as $id => $err) {
					Manspasts_Helper::add_msg('Order #' . $id . ' - ' . $err, 'error');
				}
				wp_safe_redirect(wp_get_referer());
			}
		}

		function get_terminal_address($terminal_id) {
			$terminals_json_file_dir = Manspasts_DIR . '/' . "locations.json";
			$terminals_file = fopen($terminals_json_file_dir, "r");
			$terminals = fread($terminals_file, filesize($terminals_json_file_dir) + 10);
			fclose($terminals_file);
			$terminals = json_decode($terminals, true);
			$parcel_terminals = '';
			if (is_array($terminals) && $terminal_id) {
				foreach ($terminals as $terminal) {
					if ($terminal['code'] == $terminal_id) {
						return $terminal['title'] . ', ' . $terminal['region'] . ', ' . preg_replace("/[^a-zA-Z0-9]+/", "", $terminal['code']);
					}
				}
			}
			return $terminal_id . '';
		}
	}
}
