<?php
if ( ! defined('ABSPATH') ) {
  exit; // Exit if accessed directly
}

// Prepare
$shipping_settings = Manspasts_Core::get_settings();
$configs = Manspasts_Core::get_configs();
$page_params = Manspasts_Manifest::page_params();

$orders_data = Manspasts_Manifest::page_get_orders();
$selected_orders = array();
if ( isset($_COOKIE['manspasts_checked']) ) {
  $cookie_value = json_decode(stripslashes($_COOKIE['manspasts_checked']));
  if (is_array($cookie_value)) {
    $selected_orders = $cookie_value;
  }
}

$manifest_enabled = (!isset($shipping_settings['manifest_enable']) || $shipping_settings['manifest_enable'] === 'yes') ? true : false;

// Append custom css and js
do_action('manspasts_admin_manifest_head');
?>

<div class="wrap page-manspasts_manifest">
  <h1><?php _e('Manspasts shipping', 'manspats'); ?></h1>

      <!--<div class="call-courier-container">
        <button id="manspasts-call-btn" class="button action"><?php _e('Manspasts courier', 'manspats') ?></button>
      </div>-->

      <ul class="nav nav-tabs">
        <?php foreach ( $page_params['strings'] as $tab => $tab_title ) : ?>
          <li class="nav-item">
            <a class="nav-link <?php echo $orders_data['action'] == $tab ? 'active' : ''; ?>" href="<?php echo Manspasts_Manifest::page_make_link(array('paged' => ($orders_data['action'] == $tab ? $orders_data['paged'] : 1), 'action' => $tab)); ?>"><?php echo $tab_title; ?></a>
          </li>
        <?php endforeach; ?>
      </ul>

      <?php if ( $orders_data['links'] ) : ?>
        <div class="tablenav">
          <div class="tablenav-pages">
            <?php echo $orders_data['links']; ?>
          </div>
        </div>
      <?php endif; ?>
      <?php if ( $orders_data['is_orders'] ) : ?>
        <div class="mass-print-container">
          <form id="manifest-print-form" action="admin-post.php" method="GET">
            <input type="hidden" name="action" value="manspasts_manifest" />
            <?php wp_nonce_field('manspasts_manifest', 'manspasts_manifest_nonce'); ?>
          </form>
          <form id="labels-print-form" action="admin-post.php" method="GET">
            <input type="hidden" name="action" value="manspasts_labels" />
            <?php wp_nonce_field('manspasts_labels', 'manspasts_labels_nonce'); ?>
          </form>
          <?php $desc = ''; ?>
          <div id="selected-orders" class="selected-orders <?php echo ($desc) ? 'has-desc' : ''; ?>" style="<?php echo (empty($selected_orders)) ? 'display:none' : ''; ?>">
            <span class="title"><?php echo __('Selected', 'post'); ?><?php echo ($desc) ? '*' : ''; ?>:</span>
            <?php foreach ($selected_orders as $order_id) : ?>
              <span class="item" data-id="<?php echo $order_id; ?>"><?php echo '#' . $order_id; ?><span class="dashicons dashicons-no"></span></span>
            <?php endforeach; ?>
            <?php if ($desc) : ?>
              <span class="desc">*<?php echo $desc; ?></span>
            <?php endif; ?>
          </div>
          <button id="submit_manifest_items_1" title="<?php echo __('Generate manifest', 'post'); ?>" type="button" class="button action">
                <?php echo __('Generate manifest', 'post'); ?>
              </button>
        </div>
      <?php endif; ?>

      <div class="table-container">
        <form id="filter-form" class="" action="<?php echo Manspasts_Manifest::page_make_link(array('action' => $orders_data['action'])); ?>" method="POST">
          <?php wp_nonce_field('manspasts_labels', 'manspasts_labels_nonce'); ?>
          <table class="wp-list-table widefat fixed striped posts">
            <thead>

              <tr class="manspasts-filter">
                <td class="manage-column column-cb check-column"><input type="checkbox" class="check-all" /></td>
                <th class="manage-column column-order_id">
                  <input type="text" class="d-inline" name="filter_id" id="filter_id" value="<?php echo $orders_data['filters']['id']; ?>" placeholder="<?php echo __('ID', 'manspats'); ?>" aria-label="Order ID filter">
                </th>
                <th class="manage-column">
                  <input type="text" class="d-inline" name="filter_customer" id="filter_customer" value="<?php echo $orders_data['filters']['customer']; ?>" placeholder="<?php echo __('Customer', 'manspats'); ?>" aria-label="Order ID filter">
                </th>
                <th class="column-order_status">
                  <select class="d-inline" name="filter_status" id="filter_status" aria-label="Order status filter">
                    <option value="-1" selected><?php echo _x('All', 'All status', 'manspats'); ?></option>
                    <?php foreach ( $orders_data['statuses'] as $status_key => $status ) : ?>
                      <option value="<?php echo $status_key; ?>" <?php echo ($status_key == $orders_data['filters']['status'] ? 'selected' : ''); ?>><?php echo $status; ?></option>
                    <?php endforeach; ?>
                  </select>
                </th>
                <th class="column-order_date">
                </th>
                <th class="manage-column">
                </th>
                <th class="manage-column">
                  <input type="text" class="d-inline" name="filter_barcode" id="filter_barcode" value="<?php echo $orders_data['filters']['barcode']; ?>" placeholder="<?php echo __('Barcode', 'manspats'); ?>" aria-label="Order barcode filter">
                </th>
                <?php if ($manifest_enabled) : ?>
                  <th class="column-manifest_date">
                    <div class='datetimepicker'>
                      <div>
                        <input name="filter_start_date" type='text' class="" id='datetimepicker1' data-date-format="YYYY-MM-DD" value="<?php echo $orders_data['filters']['start_date']; ?>" placeholder="<?php echo __('From', 'manspats'); ?>" autocomplete="off" />
                      </div>
                      <div>
                        <input name="filter_end_date" type='text' class="" id='datetimepicker2' data-date-format="YYYY-MM-DD" value="<?php echo $orders_data['filters']['end_date']; ?>" placeholder="<?php echo __('To', 'manspats'); ?>" autocomplete="off" />
                      </div>
                    </div>
                  </th>
                <?php endif; ?>
                <th class="manage-column">
                  <div class="manspasts-action-buttons-container">
                    <button class="button action" type="submit"><?php echo __('Filter', 'manspats'); ?></button>
                    <button id="clear_filter_btn" class="button action" type="submit"><?php echo __('Reset', 'manspats'); ?></button>
                  </div>
                </th>
              </tr>

              <tr class="table-header">
                <td class="manage-column column-cb check-column"></td>
                <th scope="col" class="column-order_id"><?php echo __('ID', 'manspats'); ?></th>
                <th scope="col" class="manage-column"><?php echo __('Customer', 'manspats'); ?></th>
                <th scope="col" class="column-order_status"><?php echo __('Order Status', 'manspats'); ?></th>
                <th scope="col" class="column-order_date"><?php echo __('Order Date', 'manspats'); ?></th>
                <th scope="col" class="manage-column"><?php echo __('Service', 'manspats'); ?></th>
                <th scope="col" class="manage-column"><?php echo __('Barcode', 'manspats'); ?></th>
                <th scope="col" class="column-manifest_date"><?php echo __('Manifest date', 'post'); ?></th>
                <th scope="col" class="manage-column"><?php echo __('Actions', 'manspats'); ?></th>
                <th></th>
              </tr>

            </thead>
            <tbody>
              <?php $date_tracker = false; ?>
              <?php foreach ( $orders_data['orders'] as $order ) : ?>
                <?php
                $barcode = $order->get_meta($configs['meta_keys']['barcode']);
                $manifest_date = $order->get_meta($configs['meta_keys']['manifest_date']);
                $date = date('Y-m-d H:i', strtotime($manifest_date));
                $manifest_date_old = $order->get_meta($configs['meta_keys']['manifest_date_old']); // Compatible with old
                $date_old = date('Y-m-d H:i', strtotime($manifest_date_old));
                ?>
                <?php if ( $orders_data['action'] == 'completed_orders' && $date_tracker !== $date && $date_tracker !== $date_old ) : ?>
                  <tr>
                    <?php $colspan = ($manifest_enabled) ? 9 : 8; ?>
                    <td colspan="<?php echo $colspan; ?>" class="manifest-date-title">
                      <?php echo $date_tracker = $date; ?>
                    </td>
                  </tr>
                <?php endif; ?>
                <tr class="data-row">
                  <?php $checked = (in_array($order->get_id(), $selected_orders)) ? 'checked' : ''; ?>
                  <th scope="row" class="check-column"><input type="checkbox" name="items[]" class="manifest-item" value="<?php echo $order->get_id(); ?>" <?php echo $checked; ?>/></th>
                  <td class="manage-column column-order_id">
                    <a href="<?php echo $order->get_edit_order_url(); ?>">#<?php echo $order->get_order_number(); ?></a>
                  </td>
                  <td class="column-order_number">
                    <div class="data-grid-cell-content">
                      <?php echo $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(); ?>
                    </div>
                  </td>
                  <td class="column-order_status">
                    <div class="data-grid-cell-content">
                      <?php echo wc_get_order_status_name($order->get_status()); ?>
                    </div>
                  </td>
                  <td class="column-order_date">
                    <div class="data-grid-cell-content">
                      <?php echo $order->get_date_created()->format ('Y-m-d H:i:s'); ?>
                    </div>
                  </td>
                  <td class="manage-column">
                    <div class="data-grid-cell-content">
                      <?php Manspasts_Order::admin_order_display($order, false); ?>
                    </div>
                  </td>
                  <td class="manage-column">
                    <div class="data-grid-cell-content">
                      <?php if ( $barcode ) : ?>
                        <?= $barcode; ?>
                      <?php endif; ?>
                      <?php $error = $order->get_meta($configs['meta_keys']['error']); ?>
                      <?php if ( $error ) : ?>
                        <?php if ( $barcode ) : ?><br /><?php endif; ?>
                        <span><?php echo '<b>' . __('Error', 'manspats') . ':</b> ' . $error; ?></span>
                      <?php endif; ?>
                    </div>
                  </td>
                  <td class="column-manifest_date">
                    <div class="data-grid-cell-content">
                      <?php echo $manifest_date . $manifest_date_old; ?>
                    </div>
                  </td>
                  <td class="manage-column">
                    <a href="admin-post.php?action=manspasts_labels&post=<?php echo $order->get_id(); ?>" class="button action">
                      <?php echo _x('Print', 'button', 'manspasts'); ?>
                    </a>
                    <?php if ( $barcode  && !$manifest_date) : ?>
                      <a href="admin-post.php?action=manspasts_labels&post=<?php echo $order->get_id(); ?>&process=regenerate" class="button action">
                        <?php echo _x('Regenerate', 'button', 'manspats'); ?>
                      </a>
                    <?php endif; ?>
                  </td>
                </tr>
              <?php endforeach; ?>

              <?php if ( ! $orders_data['orders'] ) : ?>
                <tr>
                  <td colspan="9">
                    <?php echo __('No orders found', 'woocommerce'); ?>
                  </td>
                </tr>
              <?php endif; ?>
            </tbody>
          </table>
        </form>
      </div>


      <script>
        jQuery('document').ready(function($) {
          // "From" date picker
          $('#datetimepicker1').datetimepicker({
            pickTime: false,
            useCurrent: false
          });
          // "To" date picker
          $('#datetimepicker2').datetimepicker({
            pickTime: false,
            useCurrent: false
          });

          // Set limits depending on date picker selections
          $("#datetimepicker1").on("dp.change", function(e) {
            $('#datetimepicker2').data("DateTimePicker").setMinDate(e.date);
          });
          $("#datetimepicker2").on("dp.change", function(e) {
            $('#datetimepicker1').data("DateTimePicker").setMaxDate(e.date);
          });

          // Pass on filters to pagination links
          $('.tablenav-pages').on('click', 'a', function(e) {
            e.preventDefault();
            var form = document.getElementById('filter-form');
            form.action = e.target.href;
            form.submit();
          });

          // Filter cleanup and page reload
          $('#clear_filter_btn').on('click', function(e) {
            e.preventDefault();
            $('#filter_id, #filter_customer, #filter_barcode, #datetimepicker1, #datetimepicker2').val('');
            $('#filter_status').val('-1');
            document.getElementById('filter-form').submit();
          });
        });
      </script>
</div>
